#
# $Id: Contig.pm 237 2013-12-27 17:25:56Z gouzy $
#

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

package Contig;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;
use strict;
use warnings;
use Digest::MD5 qw(md5 md5_hex md5_base64);
use ContigRegistry;

my %H_SCAFF2EXT    = ( "p p" => "3 5", "p m" => "3 3", "m p" => "5 5", "m m" => "5 3" );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 237 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

#Transfert de l'ancre sur le premier scaffold ?

sub _Init
{
my ($self,%h_param) = @_;

#	my @a_expected = ();
#	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init();

	$self -> _Set ('_len', 0);
	$self -> _Set ('_strand', '');
	$self -> _Set ('_name', '');
	$self -> _Set ('_rh_anchors', undef);	 # je vais garder les coordonnees cle=anchor  val=pos
	# containers de contig
	$self -> _Set ('_contigregistry', undef);# le registry qui contient toutes les instances d'un Contig
	$self -> _Set ('_rh_scaffolds', undef);  # le scaffold

	foreach my $param (keys(%h_param))
	{
		if ( $param eq 'str' )
		{
			$self->Set ($h_param{$param});
		}
		else
		{
			$self -> _Set ("_$param", $h_param{$param});
		}
	}
	my $test = $self->Get();
	if ($test !~ /^#\S+[pm]\d+$/ )
	{
		Carp::croak("Invalid Contig initialisation: either str or name+len+strand expected\n".join(' ',%h_param)."\n");
	}
}


sub Get
{
my($self)=@_;

	return "#".$$self{_name}.$$self{_strand}.$$self{_len};
}


sub Set
{
my($self,$str)=@_;
		
	if ( $str =~ /#(\S+)([pm])(\d+)/ )
	{
		$$self{_name}=$1;
		$$self{_strand}=$2;
		$$self{_len}=$3;
		return;
	}
	Carp::croak("Contig::Set: >$str< invalid format (expected=".'#(\S+)([pm])(\d+)'.")\n");
}


sub GetArray
{
my($self)=@_;

	return ($$self{_name},$$self{_strand},$$self{_len});
}


sub GetName
{
my($self)=@_;

	return $$self{_name};
}


sub SetName
{
my($self,$str)=@_;

	$$self{_name}=$str;
}


sub GetLength
{
my($self)=@_;

	return $$self{_len};
}


sub SetLength
{
my($self,$len)=@_;

	$$self{_len}=$len;
}


sub GetStrand
{
my($self)=@_;

	return $$self{_strand};
}


sub SetStrand
{
my($self,$strand)=@_;

	$$self{_strand}=$strand;
}

sub GetAnchorsFromRegistry
{
my($self) = @_;

	my $o_contigregistry = $self->_FastOneGet('_contigregistry');
	return undef if ( ! defined($o_contigregistry) );
	return undef if ( $o_contigregistry->IsRepeat() );	 # si le contig est identifie comme repeat je ne renvoie pas d'ancre
	return $o_contigregistry->GetAnchors();
}

sub GetScaffoldsIdsFromRegistry
{
my($self) = @_;

	my $o_contigregistry = $self->_FastOneGet('_contigregistry');
	return undef if ( ! defined($o_contigregistry) );
	return $o_contigregistry->GetScaffoldsIds();
}

sub GetNumberOfAnchorsFromRegistry
{
my($self)=@_;

	my $o_contigregistry = $self->_FastOneGet('_contigregistry');
	return 0 if ( ! defined($o_contigregistry) );
	return 0 if ( $o_contigregistry->IsRepeat() );	    # si le contig est identifie comme repeat je ne renvoie pas d'ancre
	return $o_contigregistry->GetNumberOfAnchors();
}

sub GetNumberOfScaffoldsFromRegistry
{
my($self)=@_;

	my $o_contigregistry = $self->_FastOneGet('_contigregistry');
	return 0 if ( ! defined($o_contigregistry) );
	return $o_contigregistry->GetNumberOfScaffolds();
}

sub GetAnchors
{
my($self)=@_;

	my $str = '';
	my $rh_anchors = $self->_FastOneGet('_rh_anchors');
	return undef if ( ! defined($rh_anchors) );
	foreach my $anchor (sort keys(%$rh_anchors) )
	{
		$str .= "$anchor ";
	}
	$str =~ s/\s$//;
	return $str;
}


sub GetAnchor
{
my($self,$str)=@_;

	my $rh_anchors = $self->_FastOneGet('_rh_anchors');
	return undef if ( !defined($str) || ! defined($rh_anchors) || ! defined($$rh_anchors{$str}) );
	return $$rh_anchors{$str};
}


sub SetAnchor
{
my($self,$str,$pos)=@_;

	my $rh_anchors = $self->_FastOneGet('_rh_anchors');
	$$rh_anchors{$str}=$pos;
	$self->_Set('_rh_anchors',$rh_anchors);
}


sub GetNumberOfAnchors
{
my($self)=@_;

	my $rh_anchors = $self->_FastOneGet('_rh_anchors');
	return 0 if ( ! defined($rh_anchors) );
	return scalar keys(%$rh_anchors);
}

sub RemoveTemporaryScaffolds
{
my($self,$pattern)=@_;

	$pattern = "VPATH|TMP" if ( !defined($pattern) );
	my $rh_scaffolds = $self->_FastOneGet('_rh_scaffolds');
	return undef if ( ! defined($rh_scaffolds) );
	foreach my $scaff (sort keys(%$rh_scaffolds) )
	{	
		if ( $$rh_scaffolds{$scaff} =~ /$pattern/ )
		{
			delete($$rh_scaffolds{$scaff});
		}
	}

}

sub GetScaffoldsIds
{
my($self)=@_;

	my %h_lst=();
	my $rh_scaffolds = $self->_FastOneGet('_rh_scaffolds');
	return undef if ( ! defined($rh_scaffolds) );
	foreach my $scaff (sort keys(%$rh_scaffolds) )
	{	
		if ( $$rh_scaffolds{$scaff} !~ /VPATH|TMP/ && $$rh_scaffolds{$scaff} ne '')
		{
			$h_lst{$$rh_scaffolds{$scaff}}=1;
		}
	}
	return join(' ',sort keys(%h_lst));
}

sub GetScaffoldId
{
my($self,$str)=@_;

	my $rh_scaffolds = $self->_FastOneGet('_rh_scaffolds');
	return undef if ( ! defined($str) || ! defined($rh_scaffolds) || ! defined($$rh_scaffolds{$str}) );
	return $$rh_scaffolds{$str};
}

sub IsContainingScaffold
{
my($self,$o_scaff)=@_;

	my $rh_scaffolds = $self->_FastOneGet('_rh_scaffolds');
	return &FALSE if ( ! defined($o_scaff) || ! defined($rh_scaffolds) || ! exists($$rh_scaffolds{$o_scaff}) );
	return &TRUE;
}


sub SetScaffold
{
my($self,$str,$val)=@_;

	my $rh_scaffolds = $self->_FastOneGet('_rh_scaffolds');
	$$rh_scaffolds{$str}=$val;
	$self->_Set('_rh_scaffolds',$rh_scaffolds);
}

sub DelAnchor
{
my($self,$str)=@_;

	my $rh_anchors = $self->_FastOneGet('_rh_anchors');
	return if (! defined($rh_anchors) || !defined($$rh_anchors{$str}));
	delete($$rh_anchors{$str});
}

sub DelScaffold
{
my($self,$str)=@_;

	my $rh_scaffolds = $self->_FastOneGet('_rh_scaffolds');
	return if (! defined($rh_scaffolds) || !defined($$rh_scaffolds{$str}));
	delete($$rh_scaffolds{$str});
	# je suis oblige de supprimer les ancres car je l'ai peut etre propage a partir de l'ancrage du scaffold
	# non trop d'impact $self->_Set('_rh_anchors',undef);
}


sub GetNumberOfScaffolds
{
my($self)=@_;

	my $rh_scaffolds = $self->_FastOneGet('_rh_scaffolds');
	return 0 if ( ! defined($rh_scaffolds) );
	my $cpt = 0;
	foreach my $scaff (keys(%$rh_scaffolds) )
	{	
		$cpt++ if ( $$rh_scaffolds{$scaff} !~ /VPATH|TMP/ );
	}
	return $cpt;
}


sub Reverse
{
my($self)=@_;

	$$self{_strand}=( $$self{_strand} eq 'p' ) ? 'm' : 'p';
}


sub ConvertPathLink2GraphLink
{
my($self,$o_tgt,$reorder)=@_;

	my $lstrand = $self->GetStrand();
	my $lid     = $self->GetName();
	my $rstrand = $o_tgt->GetStrand();
	my $rid     = $o_tgt->GetName();

	my ($srcext,$nextext) = split(' ',$H_SCAFF2EXT{"$lstrand $rstrand"});

	if ( defined($reorder) && $reorder )
	{
		if ( $lid gt $rid )
		{
			return($rid,$nextext,$lid,$srcext);
		}
	}
	return($lid,$srcext,$rid,$nextext);
}


sub AssertParams
{
my ($self,$rh_param,$ra_expected) = @_;
	
	foreach my $param (@{$ra_expected})
	{
		if ( !exists($$rh_param{$param}) )
		{
			print Dumper(%$rh_param);
			Carp::confess("Fatal error in constructor: $param must be provided\n") 
		}
	}
}


sub Dump
{
my($self)=@_;

	print Dumper($self);
}


sub _Print
{
my($self,$msg)=@_;

	print STDERR "$msg";
}



1;
