#
# $Id: HSPSet.pm 519 2016-08-26 05:40:19Z gouzy $
#

package HSPSet;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
#use GraphViz;
#use Graph::Undirected;
use Data::Dumper;
use LinkSet;
use Devel::Size qw(size total_size);



# ATTENTION: dans LinearizeScaff, le supprime les shortcuts mais les shortcuts pourraient aussi servir a gerer le repositionnement des repeats
# est ce que j'utilise encore ?


# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 519 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ('infile' );
	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init();

	foreach my $param (keys(%h_param))
	{
		$self -> _Set ("_$param", $h_param{$param});
	}

	my %h_keys =();
	keys(%h_keys) = 200000000;

	$self -> _Set ('_rh_hspds', \%h_keys);
	my $o_linkds = New LinkSet();
	$self -> _Set ('_o_linkds', $o_linkds);
}

=head2 procedure Load

 Title        : Load
 Usage        : $o_hspds -> Load;

=cut

sub Load 
{
	my ($self) = @_;
	Carp::croak('abstract method' );
}


sub AssertParams
{
my ($self,$rh_param,$ra_expected) = @_;
	
	foreach my $param (@{$ra_expected})
	{
		Carp::croak("Fatal error in constructor: $param must be provided\n") if ( !defined($$rh_param{$param}) );
	}
}

sub Add
{
my($self,$key,$val) = @_;

	$self->{_rh_hspds}->{$key} .= "$val##";
}

sub Set
{
my($self,$key,$val) = @_;

	$self->{_rh_hspds}->{$key} = "$val##";
}


sub AddIfBest
{
my($self,$key,$val,$score) = @_;

	my $best = 0;
	#best=1 equal
	#best=2 >
	my $rh_hspds = $self->{_rh_hspds};
	if ( ! defined($rh_hspds->{$key}) )
	{
		$self->Add($key,$val);
		return 0;
	}
	foreach my $entry (split(/##/,$rh_hspds->{$key}))
	{
		my $prevscore = $self->GetGlobalScore($entry);
		if ($prevscore<$score)
		{
			$self->Set($key,$val);
			return 2;
		}
		elsif ( $prevscore == $score )
		{
			$self->Add($key,$val);
			return 1;
		}
	}
	return 0;
}


sub MergeWithOrReplaceBest
{
my($self,$key,$val,$score,$splicedtarget) = @_;

	my $best = 0;
	#best=1 equal
	#best=2 >
	my $rh_hspds = $self->{_rh_hspds};
	if ( ! defined($rh_hspds->{$key}) )
	{
		$self->Add($key,$val);
		return 0;
	}
	my %h_param = ();
	$h_param{overlap}       = $self->_FastOneGet('_overlap');
	if ( $splicedtarget) # genome vs transcript
	{
		$h_param{maxgenomespan} = $self->_FastOneGet('_maxgenomespan');
	}
	else
	{
		$h_param{maxdifflength} = $self->_FastOneGet('_maxdifflength');
	}
	my $minchaindensity = $self->_FastOneGet('_minchaindensity');
	if (  defined($minchaindensity) )
	{
		$h_param{minchaindensity} = $minchaindensity;
	}
if ( 0 )
{
	{
		Carp::croak("key $key must be defined and not empty") if ( !defined($h_param{$key}) && $h_param{$key} eq '' );
	}
}

	foreach my $entry (split(/##/,$rh_hspds->{$key}))
	{
		my $prevscore = $self->GetGlobalScore($entry);
		if ($prevscore<$score)
		{
			$self->Set($key,$val);
			return 2;
		}
		elsif ( $prevscore == $score )
		{
			#"Egalite: je ne sais pas quoi faire! donc je ne fais rien\n";
			#$self->Add($key,$val);
			return 1;
		}
		else
		{
			# when hits are compatibles the newentry contains new coordinates on target and query sequence (MinMin,MaxMax)
			my ($merged,$newentry) = $self->MergeCompatibleHSPs($entry,$val,$splicedtarget,%h_param);
	#		print STDERR ">>1:$entry\n>>2:$val\n";
	#		print STDERR "($merged,$newentry)\n";
			if ( $merged )
			{
				$self->Set($key,$newentry);
				return 3;
			}
		}
	}
	return 0;
}


sub MergeCompatibleHSPs
{
my($self,$ent1,$ent2,$splicedtarget,%h_param)=@_;
my $merged   = 0;
my $newentry = 0;

	my %h_1 = split(/[\s=]/,$ent1);
	my %h_2 = split(/[\s=]/,$ent2);

	# brin different on ne fait rien
	return (0,'') if ( $h_1{strand} ne $h_2{strand} );

	my $ovl = $h_param{overlap};

	# meme brin mais large inclusion
	#if ( $h_2{tstart} > ($h_1{tstart}+$ovl) &&  $h_2{tend} < ($h_1{tend}+$ovl) && $h_2{qstart} > ($h_1{qstart}+$ovl) &&  $h_2{qend} < ($h_1{qend}+$ovl) )
	#20130122 je ne comprends pas pourquoi c'est $h_1{tstart}+$ovl et pas $h_1{tstart}-$ovl
	if ( ($h_1{tend}-$h_1{tstart}) > 10*$ovl || ($h_2{tend}-$h_2{tstart}) > 10*$ovl ) # 20170213 je ne peux pas faire ca si $ovl est de la taille des deux hits sinon c'est trop facile d'inclure
	{
		if ( $h_2{tstart} > ($h_1{tstart}-$ovl) &&  $h_2{tend} < ($h_1{tend}+$ovl) && $h_2{qstart} > ($h_1{qstart}-$ovl) &&  $h_2{qend} < ($h_1{qend}+$ovl) )
		{
		#	print STDERR "LARGE :  $h_2{tstart} > ($h_1{tstart}-$ovl) &&  $h_2{tend} < ($h_1{tend}+$ovl) && $h_2{qstart} > ($h_1{qstart}-$ovl) &&  $h_2{qend} < ($h_1{qend}+$ovl)\n";
			return (0,'') ;
		}
	}
	my %h_hsp = %h_1;
	if ( $splicedtarget)
	{
		$h_hsp{qstart} = &Min($h_1{qstart},$h_2{qstart});
		$h_hsp{qend}   = &Max($h_1{qend},$h_2{qend});
		return (0,'') if ( ($h_hsp{qend}-$h_hsp{qstart}) >  $h_param{maxgenomespan} );
		$h_hsp{tstart} = &Min($h_1{tstart},$h_2{tstart});
		$h_hsp{tend}   = &Max($h_1{tend},$h_2{tend});
	}
	else
	{
		# the length should be identical on both sequences .. but there are 'N' 
		# avant de merger il faut verifier que la distance est a peu pret la meme sur les deux sequences
		# pour les hits genomique je pourrais verifier la compatibilite mais je prefere le garder separe
		# pour pouvoir reutiliser plus facilement
		$h_hsp{qstart} = &Min($h_1{qstart},$h_2{qstart});
		$h_hsp{qend}   = &Max($h_1{qend},$h_2{qend});
		$h_hsp{tstart} = &Min($h_1{tstart},$h_2{tstart});
		$h_hsp{tend}   = &Max($h_1{tend},$h_2{tend});
		my $tspan = $h_hsp{tend}-$h_hsp{tstart}+1;
		my $qspan = $h_hsp{qend}-$h_hsp{qstart}+1;
		my($deltat,$deltaq);
		#print STDERR "DELTA $delta abs(($h_hsp{qstart}-$h_hsp{qend}) - ($h_hsp{tstart}-$h_hsp{tend})) $ent1 $ent2\n";
		# 20151023
		# plutot qu'un delta global, je voudrais que ce soit un delta par rapport a la position la plus proche
		if ( $h_1{qstart} < $h_2{qstart} )
		{
			$deltaq = $h_2{qstart}-$h_1{qend};
		}
		else
		{
			$deltaq = $h_1{qstart}-$h_2{qend};
		}
		#$deltaq=0 if ( $deltaq < 0 );
		if ( $h_1{tstart} < $h_2{tstart} )
		{
			$deltat = $h_2{tstart}-$h_1{tend};
		}
		else
		{
			$deltat = $h_1{tstart}-$h_2{tend};
		}
		#$deltat=0 if ( $deltat < 0 );
#		print STDERR "$deltaq // $deltat\n";
		my $delta = abs($deltaq - $deltat);
		if ( $delta > $h_param{maxdifflength} )
		{
#			print STDERR "$delta > maxdifflength($h_param{maxdifflength})\n";
			return (0,'') 
		}
		# repeat en tandem sur target
		# le span sur la target est < a la longueur des deux hits + overlap autorise
		my $tovl = $tspan-($h_1{tend}-$h_1{tstart})+1-($h_2{tend}-$h_2{tstart})+1;
		if ( $tovl < -1 * $ovl && $splicedtarget )  # 20160826: sur des genomes tres proches megablast donne des hit tres chevauchant
		{
			#print STDERR "$tovl = $tspan-($h_1{tend}-$h_1{tstart})+1-($h_2{tend}-$h_2{tstart})+1\n";
			#print STDERR "ovl : $tovl / $ovl\n";
			return (0,''); 
		}
	}
	$h_hsp{len}    = $h_hsp{tend}-$h_hsp{tstart}+1;
	my $l = ( $h_hsp{len} > ( $h_2{len} + $h_1{len} ) ) ? ( $h_2{len} + $h_1{len} ) : $h_hsp{len};
	$h_hsp{pci}    = sprintf("%.2f",&Min(($h_1{pci}*$h_1{len}+$h_2{pci}*$h_2{len})/$l,100));
	# 20120616 $h_hsp{score}  = $h_hsp{pci};
	$h_hsp{score}  = int($l*2*$h_hsp{pci}/100); # pour blastn il faut ~ multiplier par 2 pour approximer
	# overestimated
	$h_hsp{gap}    = $h_1{gap}+$h_2{gap};
	$h_hsp{mm}     = $h_1{mm}+$h_2{mm};
	if ( defined($h_param{minchaindensity}) ) # 20161006 lorsque l'on chaine quelque chose de tres proche il faut se proteger de hits lointains compatibles
	{
		if ( ($l/$h_hsp{len}) < $h_param{minchaindensity} )
		{
		#printf STDERR "$l/$h_hsp{len}) < $h_param{minchaindensity}\n";
			return (0,''); 
		}
	}
	my $val = $self->GetEntry(\%h_hsp);
	return (1,$val);
	
}

sub GetGlobalScore
{
my($self,$entry) = @_;

	my ($len) = $entry =~ /len=(\d+)/;
	my ($pci) = $entry =~ /pci=(\S+)/;

	return $len*$pci;
}


sub Get
{
my($self,$key) = @_;

	return $self->{_rh_hspds}->{$key};
}

sub GetHsps
{
my($self) = @_;

	return keys(%{$self->{_rh_hspds}});
}


sub DumpM8
{
my($self,$f_stream)=@_;

	my $querycol = $self->_FastOneGet('_query_col');
	my $rh_hspds = $self->{_rh_hspds};
	my %h_hsp=();
	my $refq = $self->_FastOneGet('_query');
	foreach my $key (sort keys(%{$rh_hspds}))
	{
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			%h_hsp =();
			%h_hsp = split(/[\s=]/,$entry);
			#my @a_fields = split(/\t/,$entry);	# query=val\ttarget=val\t...
			#foreach my $field (@a_fields)
			#{
			#	my @a_keyval = split('=',$field);
			#	$h_hsp{$a_keyval[0]} = $a_keyval[1];
			#}
			if ( $querycol == 2 )
			{
				my ($qstart,$qend) = ($h_hsp{strand} eq '+' ) ? ($h_hsp{qstart},$h_hsp{qend}) : ($h_hsp{qend},$h_hsp{qstart});
				printf $f_stream "$h_hsp{target}\t$h_hsp{query}\t$h_hsp{pci}\t$h_hsp{len}\t$h_hsp{mm}\t$h_hsp{gap}\t$h_hsp{tstart}\t$h_hsp{tend}\t$qstart\t$qend\t$h_hsp{'e-value'}\t%d\n",$h_hsp{score}; # printf pour passer en notation decimale si on arrive en notation scientifique
			}
			else
			{
				my ($tstart,$tend) = ($h_hsp{strand} eq '+' ) ? ($h_hsp{tstart},$h_hsp{tend}) : ($h_hsp{tend},$h_hsp{tstart});
				printf $f_stream "$h_hsp{query}\t$h_hsp{target}\t$h_hsp{pci}\t$h_hsp{len}\t$h_hsp{mm}\t$h_hsp{gap}\t$h_hsp{qstart}\t$h_hsp{qend}\t$tstart\t$tend\t$h_hsp{'e-value'}\t%d\n",$h_hsp{score};
			}
		}
	}
}

sub DumpLinks
{
my($self,$f_stream,$f_png)=@_;

	my $querycol = $self->_FastOneGet('_query_col');
	my $rh_hspds = $self->{_rh_hspds};
	my %h_hsp=();
	my $refq = $self->_FastOneGet('_query');
	my $pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	my $reft = $self->_FastOneGet('_target');
	my $insert_size = $self->_FastOneGet('_target')->_FastOneGet('_insert_size');
	my $delta = $insert_size + $insert_size/4;
	$self->_Print("delta=$delta, pattern=$pattern\n");
	foreach my $param ( $pattern, $insert_size )
	{
		if ( !defined($param) || ! $param )
		{
			Carp::croak("Fatal error: pattern and/or  insert_size undefined for dataset ".$self->_FastOneGet('_target'));
			return;
		}
	}	
	my %h_insert=();
	my $nboneend=0;
	my $nbsameseq=0;
	# identify paired ends
	foreach my $key (sort keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($query) =$key =~ /query=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		my ($qlen)   = $refq->GetLen($query);
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			%h_hsp =();
			%h_hsp = split(/[\s=]/,$entry);
			my $ok = 0;
			my ($minq,$maxq) = &MinMax($h_hsp{qstart},$h_hsp{qend});	
			my $dist = $maxq;
			if ( $maxq <= $delta )
			{
				$ok = 'Start';
			}
			$dist += ($qlen-$minq);
			if ( $qlen-$minq <= $delta )
			{
				if ( ! $ok )
				{
					$ok = 'End';
				}
				else
				{
					$ok .= 'End';
				}
			}
			if ( $ok =~ /Start|End/ )
			{
				$h_insert{$insert}{$ext} .= "$query:$ok " if ( $h_insert{$insert}{$ext} !~ /$query/ );
			}
		}
	}
	my %h_links=();
	foreach my $insert(keys(%h_insert))
	{
		my @a_ends = keys(%{$h_insert{$insert}});
		my %h_query = ();
		for(my $i=0;$i<$#a_ends;$i++)
		{
			my @a_i = split(' ',$h_insert{$insert}{$a_ends[$i]});
			for(my $j=$i+1;$j<=$#a_ends;$j++)
			{
				my @a_j = split(' ',$h_insert{$insert}{$a_ends[$j]});
				for(my $ii=0;$ii<=$#a_i;$ii++)
				{
					for(my $jj=0;$jj<=$#a_j;$jj++)
					{
						my @a_sort = sort($a_j[$jj],$a_i[$ii]);
						my $key = join("\t",@a_sort);
						$h_links{$key} .= "$insert ";
					}
				}
			}
		}
	}
	$self->_Print("nb links = ".scalar(keys(%h_links)),"\n");
	my %h_deja=();
#	my $g = GraphViz->new(directed=>0, pagewidth => 8.5, pageheight => 11, fontsize=>8);
	#my $G = Graph::Undirected->new(hyperedged=>1);
	foreach my $pair (keys(%h_links))
	{
		my @a_ = split(' ',$pair);
		my $nb = scalar split(' ',$h_links{$pair});
		my ($scaf0,$end0) = $a_[0] =~ /(\w+):(\w+)/;
		my ($scaf1,$end1) = $a_[1] =~ /(\w+):(\w+)/;
		my $limit = ( $end0 =~ /StartEnd/ || $end1 =~ /StartEnd/ ) ? 2 : 3;
		if ( $nb >= $limit )
		{
			#$G->add_vertex($scaf0);
			#$G->add_vertex($scaf1);
			#$G->add_vertex($a_[0]);
			#$G->add_vertex($a_[1]);
			#$G->add_edge($a_[0],$a_[1]);
			#$G->set_edge_attribute($a_[0],$a_[1], 'weight', $nb);

			#$g->add_node($scaf0);
			#$g->add_node($scaf1);
			#$g->add_node($a_[0]);
			#$g->add_node($a_[1]);
			#$g->add_edge($a_[0] => $a_[1], label => $nb);
			if (! defined($h_deja{"$scaf0-$a_[0]"}))
			{
				#$G->add_edge($scaf0,$a_[0]);
				#$g->add_edge($scaf0 => $a_[0]);
			}
			if (! defined($h_deja{"$scaf1-$a_[1]"}))
			{
				#$G->add_edge($scaf1,$a_[1]);
				#$g->add_edge($scaf1 => $a_[1]);
			}
			$h_deja{"$scaf0-$a_[0]"}=1;
			$h_deja{"$scaf1-$a_[1]"}=1;
		}
		printf $f_stream "$pair\t$nb\t$h_links{$pair}\n";
	}
	#print $f_png $g->as_png;
	#if ( 0 )
	#{
	#my @a_v = $G->vertices;
	#foreach my $v (@a_v)
	#{
	#	if ( $v !~ /Start|End/ && $G->vertex_degree($v) == 1 )
	#	{
	#		$G->delete_vertex($v);
	#	}
	#}
	#}
	#my @a_cc = $G->connected_components();
	#foreach my $cc (@a_cc)
	#{
	#	print $f_stream Dumper($cc);
	#}
}


sub PrintInfo
{
my($self) = @_;

	my $refq = $self->_FastOneGet('_query');
	my $reft = $self->_FastOneGet('_target');

	my $rh_hspds = $self->{_rh_hspds};
	my $nb_keys = keys(%{$rh_hspds});
	$self->_Print("Number of query-target: $nb_keys\n");
}

sub GetEntry
{
my ($self,$rh_hsp) = @_;

	my $val = '';
	foreach my $k (sort keys(%$rh_hsp))
	{
		$val .= "$k=$$rh_hsp{$k}\t";
	}
	return $val;
}

sub InitMinMax
{
my($self,$rh_min,$rh_max,@a_criteria)=@_;

	foreach my $criterium (@a_criteria)
	{
		$$rh_min{$criterium} = ( $self->_FastOneGet("_min$criterium") ) ? $self->_FastOneGet("_min$criterium") : 0;
		$$rh_max{$criterium} = ( $self->_FastOneGet("_max$criterium") ) ? $self->_FastOneGet("_max$criterium") : &MAXINT;
	}
}

#
#============= Filters =================
#

# on conserve tous les meilleurs scores

sub Best
{
my($self,$switch) = @_;

	$self->_Print("Filter: Best starting\n");
	my $actual_target = ( defined($switch) && $switch ) ? 'query' : 'target';
	my $rh_hspds = $self->{_rh_hspds};
	my %h_bestscore=();
	my %h_bestkey=();
	my %h_bestentry=();
	# first identify best hits for each target
	my @a_keys = keys(%{$rh_hspds});
	foreach my $key (@a_keys)
	{
		my ($target) =$key =~ /$actual_target=(\S+)/;
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			my $score = $self->GetGlobalScore($entry);
			if ( !defined($h_bestscore{$target}) || $score > $h_bestscore{$target} )
			{
				$h_bestscore{$target} = $score;
				$h_bestkey{$target}   = "$key##";
			}
			elsif ( $score == $h_bestscore{$target} )
			{
				$h_bestkey{$target}   .= "$key##";
			}
		}
	}
	# second pass, cleaning the data structure
	my $nbentries=0;
	my $nbkeys=0;
	foreach my $key (@a_keys)
	{
		my ($target)=$key =~ /$actual_target=(\S+)/;
		my $tmpkey = $key;
		$tmpkey =~ s/\|/\\|/g;
		$tmpkey =~ s/\./\\./g;
		if (! defined($h_bestkey{$target}) || $h_bestkey{$target} !~ /$tmpkey/ )
		{
			#print STDERR " deletion of $key : $h_bestkey{$target}\n";
			delete($rh_hspds->{$key});
			next;
		}
		$nbkeys++;
		my $newentry = '';
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			my $score = $self->GetGlobalScore($entry);
			if ( $score == $h_bestscore{$target} )
			{
				$newentry .= "$entry##";
				$nbentries++;
			}
		}
		if ( $newentry eq '' )		# c'est une trace car il doit toujours y avoir au moins un meilleur
		{
			foreach my $entry (split(/##/,$rh_hspds->{$key}))
			{
				my $score = $self->GetGlobalScore($entry);
				$self->_Print("$key: score=$score, $actual_target=$target, best=$h_bestscore{$target}\n");
				if ( $score == $h_bestscore{$target} )
				{
					$newentry .= "$entry##";
					$nbentries++;
				}
			}
			Carp::croak("Fatal error:\n$key\n\t$rh_hspds->{$key}\n");
		}
		$self->{_rh_hspds}->{$key} = "$newentry";
	}
	$self->_Print("Filter: Best done\n");
	return($nbkeys,$nbentries);
}


sub Covt
{
my($self,$r_tseq) = @_;

	#print STDERR Dumper($r_tseq);
	#print STDERR "($self,$r_tseq)\n";
	$self->_Print("Filter: Covt starting\n");
	my $mincovt = $self->_FastOneGet('_mincovt');
	my $maxcovt = $self->_FastOneGet('_maxcovt');
	my $rh_hspds = $self->{_rh_hspds};
	# identify target overlapping the query 
	# short hits or inclusions are removed
	# (mincovt < overlap < maxcovt)
	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			my ($hsplen) = $entry =~ /len=(\d+)/;
			my ($slen)   = $r_tseq->GetLen($target);
			if ( !defined($slen) || $slen == 0 )
			{
				Carp::croak("Error: Covt: can't get length for >$target< sequence\n");
			}
			my $covt = $hsplen*100 /$slen;
			$covt = 100 if ( $covt > 100);
			if ( $covt < $mincovt || $covt > $maxcovt )	
			{
				delete($rh_hspds->{$key});
			}
		}
	}
	$self->_Print("Filter: Covt done\n");
}

sub NoSelf
{
my($self) = @_;

        $self->_Print("Filter: NoSelf starting\n");
        my $rh_hspds = $self->{_rh_hspds};
        foreach my $key (sort keys(%{$rh_hspds}))
        {
                my ($target)=$key =~ /target=(\S+)/;
                my ($query)= $key =~ /query=(\S+)/;
                if ($target eq $query )
                {
                        delete($rh_hspds->{$key});
                }
        }
        $self->_Print("Filter: NoSelf done\n");
}


# je n'analyse pas les entries, c'est un QueryCount

sub QueryCount
{
my($self) = @_;

	my %h_min=();
	my %h_max=();
	$self->InitMinMax(\%h_min,\%h_max,'count');

	$self->_Print("Filter: QueryCount starting\n");
	$self->_Print("\tmincount=$h_min{count}, maxcount=$h_max{count}\n");
	
	my %h_target = ();
	
	my $rh_hspds = $self->{_rh_hspds};
	# first pass, count
	my @a_keys = keys(%{$rh_hspds});
	foreach my $key (@a_keys)
	{
		my ($target)=$key =~ /target=(\S+)/;
		$h_target{$target}++;
	}

	# second pass, removal of low/high counts
	foreach my $key (@a_keys)
	{	
		my ($target)=$key =~ /target=(\S+)/;
		if( ($h_target{$target} < $h_min{'count'}) || ($h_target{$target} > $h_max{'count'}) )
		{
			delete($rh_hspds->{$key});
		}	
	}
	$self->_Print("Filter: QueryCount done\n");
}

# remove HSPs involving short queries (e.g for scaffolding)

sub MinQueryLen
{
my($self,$minlen,$r_qseq) = @_;

	$self->_Print("Filter: MinQueryLen starting (minlen=$minlen)\n");
	my $rh_hspds = $self->{_rh_hspds};
	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($query)=$key =~ /query=(\S+)/;
		my ($qslen)   = $r_qseq->GetLen($query);
		if ( $qslen < $minlen )
		{
			delete($rh_hspds->{$key});
		}
	}
	$self->_Print("Filter: MinQueryLen done\n");
}

#
# ne garde que sequences qui permettent de faire le lien entre deux query
# Prerequis: il faut d'abord passer QueryCount avec un mincount >= 2
# les hits doivent etre compatibles sur la target

sub Compatible
{
my($self,$r_tseq,$r_qseq) = @_;

	# les deux seuils doivent etre remplis
	my $overhang = $self->_FastOneGet('_overhang');
	my $overhang_pclen = $self->_FastOneGet('_overhang_pclen') || 5;  # 20160724
	my $remove_included = $self->_FastOneGet('_remove_included') || 0;  # 20170331
	my $test_included_compatibility = $self->_FastOneGet('_test_included_compatibility') || 0;  # 20170331

	$self->_Print("Filter: Compatible starting\n");
	$self->_Print("Filter: overhang=$overhang, overhang_pclen=$overhang_pclen, remove_included=$remove_included, test_included_compatibility=$test_included_compatibility\n");
	my $rh_hspds = $self->{_rh_hspds};
	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($tslen)   = $r_tseq->GetLen($target);
		my ($query)=$key =~ /query=(\S+)/;
		my ($qslen)   = $r_qseq->GetLen($query);
		my ($ovhq) = &Min($overhang,int($overhang_pclen*$qslen/100));
		my ($ovht) = &Min($overhang,int($overhang_pclen*$tslen/100));
		#print STDERR "($ovhq) ($ovht)\n";
		if ( $qslen !~ /\d+/ && $tslen =~ /\d+/ )
		{
			Carp::croak("Undefined length qslen=$qslen, tslen=$tslen\n");
		}
		#print "$target, $tslen ; $query, $qslen##\n";
		my $del = 0;
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			#print "\t$entry\n";
			my %h_hsp = split(/[\s=]/,$entry);
			my $deltaqmin = $h_hsp{qstart}-1;
			my $deltaqmax = $qslen-$h_hsp{qend};
			my $deltatmin = $h_hsp{tstart}-1;
			my $deltatmax = $tslen-$h_hsp{tend};
			#print STDERR "$deltaqmin $deltaqmax $deltatmin $deltatmax\n";
			my $ok = 0;
			# inclusion
			#print STDERR "( ( $deltaqmax < $ovhq &&  $deltaqmin < $ovhq ) || ( $deltatmin < $ovht && $deltatmax < $ovht ) )\n";
			if ( ( $deltaqmax < $ovhq &&  $deltaqmin < $ovhq ) || ( $deltatmin < $ovht && $deltatmax < $ovht ) )
			{
				# 20170331: rajoute pour ne garder que les included qui pourraient etre compatible avec un scaffoling
				if ( ! $remove_included )
				{
					if ( ! $test_included_compatibility )
					{
						$ok = 1;
					}
					else
					{
						# si la query est incluse je regarde que je suis a une frontiere de la target
						if ( $deltaqmax < $ovhq &&  $deltaqmin < $ovhq && ( $deltatmin < $ovht || $deltatmax < $ovht ))
						{
							$ok = 1;
						}
						elsif( $deltatmin < $ovht && $deltatmax < $ovht && ( $deltaqmax < $ovhq || $deltaqmin < $ovhq )) # et si c'est la target je dois etre a une frontiere de la query
						{
							
							$ok = 1;
						}
					}
				}
			}
			elsif ( $h_hsp{strand} eq '+' )
			{
				# query: 19210,  1
				# target: 0,  7538
				# or
				# query: 8,  8279
        		# target: 1307,  0

				if ( ( $deltaqmax < $ovhq && $deltatmin < $ovht ) || ( $deltaqmin < $ovhq && $deltatmax < $ovht ) )
				{	
					$ok = 1;
				}
			}
			else 
			{
				# query: 7487,  1
        		# target: 2027,  0
				# or 
				# query: 0,  8732
        		# target: 0,  9961

				if ( ( $deltaqmin < $ovhq && $deltatmin < $ovht ) || ( $deltaqmax < $ovhq && $deltatmax < $ovht ) )
				{	
					$ok = 1;
				}
			}
			#print "\tquery: $deltaqmin,  $deltaqmax (ovhq=$ovhq)\n";
			#print "\ttarget: $deltatmin,  $deltatmax (ovht=$ovht)\n";
			#print "\t\tOK = $ok\n";
			if ( ! $ok )
			{
				$del = 1;
			}
		}
		delete($rh_hspds->{$key}) if ( $del );
	}
	$self->_Print("Filter: Compatible done\n");
}

#
# ne garde que sequences qui permettent de faire le lien entre deux query
# Prerequis: il faut d'abord passer QueryCount avec un mincount >= 2
# les hits doivent etre compatibles sur la target
#


sub LinkBySeq
{
my($self,$r_tseq) = @_;

	$self->_Print("Filter: LinkBySeq starting\n");
	my $ovl = $self->_FastOneGet('_overlap'); # c'est un overlap sur la target
	my $minunmapped = $self->_FastOneGet('_minunmapped') || 0; # pour s'assurer que deux hits tres chevauchant amenent quand meme de l'information des deux cotes (PacBio)
	$self->_Print("Filter: overlap=$ovl, minunmapped=$minunmapped\n");
	my $rh_hspds = $self->{_rh_hspds};
	my %h_target=();
	# je collecte les donnees par target
	my @a_keys = keys(%{$rh_hspds});
	foreach my $key (@a_keys)
	{
		my ($target)=$key =~ /target=(\S+)/;
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			$h_target{$target} .= "$entry##";
		}
	}
	my %h_target2query=();
	# j'analyse les paires de hits pour voir s'ils sont compatibles sur la target
	foreach my $target (keys(%h_target))
	{
		$h_target{$target}=~s/##$//;
		my @a_   = split(/##/,$h_target{$target});
		my @a_ok = split('',0 x ($#a_+1));
		for (my $i=0; $i<$#a_;$i++)
		{
			my %h_i = split(/[\s=]/,$a_[$i]);
			for (my $j=$i+1; $j<=$#a_;$j++)
			{
				my %h_j = split(/[\s=]/,$a_[$j]);
				if ( abs($h_i{tstart}-$h_j{tstart}) >= $minunmapped && abs($h_i{tend}-$h_j{tend}) >= $minunmapped )
				{
					if ( $h_i{tstart} < $h_j{tstart} )
					{
						if ( $h_i{tend} < ($h_j{tstart}+$ovl) )
						{	
							$a_ok[$i]=1;
							$a_ok[$j]=1;
						}
					}
					else
					{
						if ( $h_j{tend} < ($h_i{tstart}+$ovl) )
						{	
							$a_ok[$i]=1;
							$a_ok[$j]=1;
						}
					}
				}
			}
		}	
		for (my $i=0;$i<=$#a_;$i++)
		{
			my ($query) = $a_[$i] =~ /query=(\S+)/;
			$h_target2query{$target} .= "query=$query " if ($a_ok[$i] == 1 );
		}
	}
	foreach my $key (@a_keys)
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($query) =$key =~ /query=(\S+)/;
		if ( !defined($h_target2query{$target}) || $h_target2query{$target} !~ /$query\s/ )
		{
			delete($rh_hspds->{$key});
		}
	}
	$self->_Print("Filter: LinkBySeq done\n");
}

sub SetEndLabels
{
my($self,$end1,$end2) = @_;

	$self->_Set ("_end1", $end1);
	$self->_Set ("_end2", $end2);
}


# identifier les 2 codes majoritaires (en cas de resequencage il y a parfois des RM1 RM2, etc)

sub InitEndLabels
{
my($self) = @_;

	return if ( defined($self->_FastOneGet('_end1')) && $self->_FastOneGet('_end1') ne '' &&  defined($self->_FastOneGet('_end2')) && $self->_FastOneGet('_end2') ne '');
	$self->_Print("InitEndLabels starting\n");
	my $rh_hspds = $self->{_rh_hspds};
	my $pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	$self->_Print("Filter: target/pattern=$pattern\n");
	my %h_ext=();
	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		$h_ext{$ext}++;
	}
	my $c=0;
	my @a_key = sort { $h_ext{$b} <=> $h_ext{$a} } keys(%h_ext);
	$self->_Set ("_end1", $a_key[0]);
	$self->_Set ("_end2", $a_key[1]);
	$self->_Print("End labels:\tend1 = ".$self->_FastOneGet('_end1')."\tend2 = ".$self->_FastOneGet('_end2')."\n");
	$self->_Print("InitEndLabels done\n");
}

# remove HSPs for which then end label is not cannonic

sub CheckEndLabels
{
my($self) = @_;

	$self->_Print("Filter: CheckEndLabels starting\n");
	my $rh_hspds = $self->{_rh_hspds};
	my $pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	$self->_Print("Filter: target/pattern=$pattern\n");
	$self->InitEndLabels();
	my $e1 = $self->_FastOneGet('_end1');
	my $e2 = $self->_FastOneGet('_end2');
	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		if ( $ext ne $e1 && $ext ne $e2 )
		{
			delete($rh_hspds->{$key});
		}
	}
	$self->_Print("Filter: CheckEndLabels done\n");
}

# ne garde que les liens compatibles avec un scaffolding
# pour le mode PE=proche des extremites et compatible avec la taille des insert
# ATTENTION: il suffit que l'on ait un couple de correct pour les garder tous, par contre seuls les compatibles sont charges dans les liens
# ==> le fichier de sortie contient tout, ca peut etre trompeur

sub CompatScaff
{
my($self) = @_;
	
	$self->_Print("Filter: CompatScaff starting\n");
	my $nb_links_loaded = 0;
	my $oriented  = $self->_FastOneGet('_query')->_FastOneGet('_oriented');
	my $mode = ( defined($oriented) && $oriented =~ /^y/ ) ? "oriented" : "";
	$mode .= " update ";
	my $pattern   = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	my $insertlen = $self->_FastOneGet('_target')->_FastOneGet('_insert_size');
	my $paired    = $self->_FastOneGet('_target')->_FastOneGet('_paired');
	$self->_Print("Filter: query/oriented=$oriented\n");
	$self->_Print("Filter: target/pattern=$pattern\n");
	$self->_Print("Filter: target/insert_size=$insertlen\n");
	$self->_Print("Filter: target/paired=$paired\n");
	if ( $pattern =~ /\S+/ && $insertlen =~ /^\d+$/ && $paired =~ /(direct|revcomp|opposite)/ )
	{
		$self->_FastOneGet('_o_linkds')->Reset();
		$nb_links_loaded = $self->_FastOneGet('_o_linkds')->LoadHSPSetByPE($self,$mode);
	}
	else
	{
		print STDERR "Warning: to date, CompatScaff can only manage links based on pe or mp\n";
	}
	if ($nb_links_loaded < 1 )
	{
		$self->_Print("No HSP found\n");
	}
	$self->_Print("nb paired links compatible with scaffolding=$nb_links_loaded\n");
	$self->_Print("CompatScaff done\n");
	return $nb_links_loaded;
}



sub LinearizeScaff
{
my($self) = @_;

	$self->_Print("Filter: LinearizeScaff starting\n");
	my $nb_links_loaded = 0;
	my $oriented  = $self->_FastOneGet('_query')->_FastOneGet('_oriented');
	my $mode = ( defined($oriented) && $oriented =~ /^y/ ) ? "oriented" : "";
	$mode .= " update ";
	my $pattern   = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	my $insertlen = $self->_FastOneGet('_target')->_FastOneGet('_insert_size');
	my $paired    = $self->_FastOneGet('_target')->_FastOneGet('_paired');
	$self->_Print("Filter: query/oriented=$oriented\n");
	$self->_Print("Filter: target/pattern=$pattern\n");
	$self->_Print("Filter: target/insert_size=$insertlen\n");
	$self->_Print("Filter: target/paired=$paired\n");
	if ( $pattern =~ /\S+/ && $insertlen =~ /^\d+$/ && $paired =~ /(direct|revcomp|opposite)/ )
	{
		$self->_FastOneGet('_o_linkds')->Reset();
		$nb_links_loaded = $self->_FastOneGet('_o_linkds')->LoadHSPSetByPE($self,$mode);
	}
	else
	{
		print STDERR "Warning: to date, LinearizeScaff can only manage links based on pe or mp\n";
	}
	if ($nb_links_loaded < 1 )
	{
		$self->_Print("No HSP found\n");
		return $nb_links_loaded;
	}
	$pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	$self->InitEndLabels();
	my $end1 = $self->_FastOneGet('_end1');
	my $end2 = $self->_FastOneGet('_end2');
	my %h_insert=();
	my $rh_hspds = $self->{_rh_hspds};

	my %h_pair_to_be_removed=();
	my @a_links=$self->_FastOneGet('_o_linkds')->GetLinks();
	my %h_path=();
	foreach my $span (@a_links)
	{
		my %h_key=split(/[=\s]/,$span);
		my %h_val=split(/[=\s]/,$self->_FastOneGet('_o_linkds')->Get($span));
		$h_path{$h_key{src}}{$h_key{tgt}}=$h_val{gaplen};
		$h_path{$h_key{tgt}}{$h_key{src}}=$h_val{gaplen};
	}
	foreach my $src (keys(%h_path))
	{
		my @a_alltgt = sort keys(%{$h_path{$src}});
		foreach my $tgt (@a_alltgt)
		{
			my $found = &FALSE;
			my $tgtlen  = $self->_FastOneGet('_query')->GetLen($tgt);
			foreach my $nextnext (@a_alltgt)
			{
				if (&Exists(\%h_path,$tgt,$nextnext)) # on peut passer de src => tgt => nextnext  (sachant src=>nextnext)
				{
					my $pair = join(' ',sort($src,$nextnext));
					next if ( defined($h_pair_to_be_removed{$pair}) );
					if ( $tgtlen < $h_path{$src}{$nextnext} )
					{
						$h_pair_to_be_removed{$pair}=1;
						$self->_Print("path $src => $tgt => $nextnext, removing pair=$pair, gapsize=$h_path{$src}{$nextnext}, $tgt len=$tgtlen < $h_path{$src}{$nextnext}\n");
						$found=&TRUE;
					}
				}
				last if($found);
			}
			last if ($found);
		}
	}

	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($query) =$key =~ /query=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		next if ( $ext ne $end1 && $ext ne $end2 );
		$h_insert{$insert}{$ext} = $$rh_hspds{$key};
	}
	foreach my $insert (keys(%h_insert))
	{
		my $h1 = $h_insert{$insert}{$end1};	
		$h1 =~ s/\s+##//;
		my $h2 = $h_insert{$insert}{$end2};
		$h2 =~ s/\s+##//;
		my %h_1 = split(/[=\s]/,$h1);
		my %h_2 = split(/[=\s]/,$h2);
		my $pair = join(' ',sort($h_1{query},$h_2{query}));
		if ( defined($h_pair_to_be_removed{$pair}) )
		{
			#$self->_Print("deletion $insert, $pair\n");
			delete($h_insert{$insert});
		}
	}
	$self->_RemoveFromListOfInsert(\%h_insert,$pattern);
	return $nb_links_loaded;
}

#
# NonRedundantPE
#

sub NonRedundantPE
{
my($self) = @_;

	$self->_Print("Filter: NonRedundantPE starting\n");
	my $pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	$self->_Print("Filter: target/pattern=$pattern\n");
	if ( $pattern =~ /^\s*$/ )
	{
		Carp::croak("Error: undefined pattern\n");
	}
	my $rh_hspds = $self->{_rh_hspds};
#	my $total_size_before = total_size($rh_hspds);
	my %h_insert_ok=();
	my %h_deja_vu = ();
	my %h_insert=();
	my $prev_insert='';
	# pour consommer moins de RAM, je tri sur la target et je fais l'analyse pour chaque insert
	foreach my $key (sort { my ($ta)=$a=~/target=(\S+)/; my ($tb)=$b=~/target=(\S+)/; return $ta cmp $tb; } keys(%{$rh_hspds})) # query=val target=val
	{
		my ($query,$target)=$key =~ /query=(\S+)\s+target=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		if ( $insert ne $prev_insert && $prev_insert ne '' )
		{
			&RemoveRedundantInserts(\%h_insert,\%h_insert_ok,\%h_deja_vu);
			%h_insert=();
		}
		#$h_insert{$insert}{$ext} .= $rh_hspds->{$key}."##";
		$h_insert{$insert}{$ext} .= $rh_hspds->{$key}; # inutile, tous les hits finissent par ##
		$prev_insert=$insert;
	}
	&RemoveRedundantInserts(\%h_insert,\%h_insert_ok,\%h_deja_vu);

	$self->_Print("NonRedundantPE #inserts post: ".keys(%h_insert_ok)."\n");
	$self->_RemoveFromListOfInsert(\%h_insert_ok,$pattern);
#	my $total_size_after = total_size($rh_hspds);
#	$self->_Print("Filter: NonRedundantPE mem_size before=$total_size_before, after=$total_size_after\n");
}

sub RemoveRedundantInserts
{
my($rh_insert,$rh_insert_ok,$rh_deja_vu)=@_;

	my $REGION_LENGTH=10;
	my $READLEN=100;
	my $SEMI_REGION_LENGTH=$REGION_LENGTH/2;
	foreach my $insert(keys(%{$rh_insert}))
	{
		my @a_ends = keys(%{$$rh_insert{$insert}});
		next if ( scalar(@a_ends) < 1 ); # only one has been mapped on this subset, skipped
		my @a_1 = split(/##/,$$rh_insert{$insert}{$a_ends[0]});
		my @a_2 = split(/##/,$$rh_insert{$insert}{$a_ends[1]});
		for(my $i=0;$i< scalar(@a_1);$i++)
		{
			my %h_1 = split(/[\s=]/,$a_1[$i]);
			#&_RecomputeTargetHits(\%h_1,$READLEN); # ca ne semble rien apporter sur les donnees eurofins
			#tant que je n'ai pas calcule l'orientation des contigs, je ne peux pas calculer le min 
			#my $st1     = ( $h_1{strand} eq '+' ) ? int(($h_1{qstart})/$REGION_LENGTH) :  int(($h_1{qend})/$REGION_LENGTH);
			# je vais prendre la moyenne
			my $mid1 = int((abs($h_1{qstart}+$h_1{qend}))/2);
			my $st1  = int($mid1/$REGION_LENGTH);
			my $mod1 = $mid1 % $REGION_LENGTH;
			#print STDERR "AV $st1 $mid1 $mod1\n";
			$st1++ if ( $mod1 >= ($SEMI_REGION_LENGTH/2) );
			#print STDERR "AP $st1\n";
			my $str1    = "$h_1{query}-$st1";
			for(my $j=0;$j< scalar(@a_2);$j++)
			{
				my %h_2 = split(/[\s=]/,$a_2[$j]);
				#&_RecomputeTargetHits(\%h_2,$READLEN);
				#my $st2 = ( $h_2{strand} eq '+' ) ? int(($h_2{qstart})/$REGION_LENGTH) : int(($h_2{qend})/$REGION_LENGTH);
				my $mid2 = int((abs($h_2{qstart}+$h_2{qend}))/2);
				my $st2  = int($mid2/$REGION_LENGTH);
				my $mod2 = $mid2 % $REGION_LENGTH;
				#print STDERR "AV $st2 $mid2 $mod2\n";
				$st2++ if ( $mod2 >= ($SEMI_REGION_LENGTH/2) );
				#print STDERR "AP $st2\n";
				my $str2= "$h_2{query}-$st2";
				my $key = join('-',sort($str1,$str2));
				#print STDERR "=== $key ===\n";
				if ( defined($$rh_deja_vu{$key}) )
				{
					#print STDERR "$key deleted by $$rh_deja_vu{$key}\n";
					#print STDERR "1: ".join(' ',@a_1)."\n";
					#print STDERR "2: ".join(' ',@a_2)."\n";
					delete($$rh_insert{$insert});
					$i=999999;
					last;
				}
				#$$rh_deja_vu{$key} = "D1: ".join(' ',@a_1)."\n"."D2: ".join(' ',@a_2)."\n";
				$$rh_deja_vu{$key} = 1;
			}
		}
	}
	foreach my $insert (keys%{$rh_insert})
	{
		$$rh_insert_ok{$insert}=1;
	}
}

# je ne peux pas savoir ou les nt manquent
sub _RecomputeTargetHits
{
my($rh_hsp,$READLEN)=@_;

	my ($qstart,$qend) = ();
	# je vais recaler les hits comme si tous les targets avaient 100nt, sinon ca ne fonctionne pas
	print STDERR "AV $$rh_hsp{target}\t$$rh_hsp{query}\t$$rh_hsp{pci}\t$$rh_hsp{len}\t$$rh_hsp{mm}\t$$rh_hsp{gap}\t$$rh_hsp{tstart}\t$$rh_hsp{tend}\t$$rh_hsp{qstart}\t$$rh_hsp{qend}\t$$rh_hsp{'e-value'}\t$$rh_hsp{score}\n";
	if ( $$rh_hsp{strand} eq '+' )
	{
		$$rh_hsp{qstart}-=$$rh_hsp{tstart}-1;
		$$rh_hsp{qend}+=&Max($READLEN-$$rh_hsp{tend},0);	# j'ai pas l'info sur la longeur ca peut depasser, tant pis. Si plus long que $READLEN je ne fais rien
	}
	else
	{
		$$rh_hsp{qstart}-=&Max($READLEN-$$rh_hsp{tend},0);
		$$rh_hsp{qend}+=$$rh_hsp{tstart}-1;
	}
	$$rh_hsp{qstart}=1 if ( $$rh_hsp{qstart} < 1 );
	($qstart,$qend) = ($$rh_hsp{strand} eq '+' ) ? ($$rh_hsp{qstart},$$rh_hsp{qend}) : ($$rh_hsp{qend},$$rh_hsp{qstart});
	print STDERR "AP $$rh_hsp{target}\t$$rh_hsp{query}\t$$rh_hsp{pci}\t$$rh_hsp{len}\t$$rh_hsp{mm}\t$$rh_hsp{gap}\t$$rh_hsp{tstart}\t$$rh_hsp{tend}\t$qstart\t$qend\t$$rh_hsp{'e-value'}\t$$rh_hsp{score}\n";
}


#
# Un filtre presque pareil que LinkByPE ou je ne garde que si l'une est spe
#

sub OneSpecificEnd
{
my($self) = @_;

	$self->_Print("Filter: OneSpecificEnd starting\n");
	my $pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	$self->_Print("Filter: target/pattern=$pattern\n");
	my $rh_hspds = $self->{_rh_hspds};
#	my $total_size_before = total_size($rh_hspds);
	my %h_insert=();
	my $nboneend=0;
	my $nbsameseq=0;
	# identify paired ends
	my @a_keys = keys(%{$rh_hspds});
	my ($key,$target,$query,$insert,$ext)=();
	my $i;
	for(my $i=0;$i<scalar(@a_keys);$i++)
	{
		$key = $$rh_hspds{$a_keys[$i]};
		($target)=$key =~ /target=(\S+)/;
		($query) =$key =~ /query=(\S+)/;
		($insert,$ext) = $target =~ /$pattern/;
		$h_insert{$insert}{$ext}{$query}=1;
	}
	$self->_Print("OneSpecificEnd #insert pre: ".scalar(keys(%h_insert))."\n");

	my @a_keys = keys(%h_insert);
	my @a_ = ();
	my @a_ends = ();
	my $min=&MAXINT;
	my %h_query=();

	my %h_insert_ok=();
	my $nok = &FALSE;

	for(my $i=0;$i<scalar(@a_keys);$i++)
	{
		$insert = $a_keys[$i];
		@a_ends = keys(%{$h_insert{$insert}});
		if ( scalar(@a_ends) != 2 ) # only one has been mapped
		{
			$nboneend++;
			#delete($h_insert{$insert});
		}
		else
		{
			%h_query = ();
			$min = &MAXINT;
			$nok=&FALSE;
			for(my $j=0;$j<2;$j++)
			{
				@a_ = keys(%{$h_insert{$insert}{$a_ends[$j]}});
				for(my $k=0;$k<scalar(@a_);$k++)
				{
					$h_query{$a_[$k]}++;	 # je degage si les deux extremites sont sur une meme query (si les query sont tres longues cela peut etre des FP
					if ( $h_query{$a_[$k]} == 2 )
					{
						$nbsameseq++;
						#delete($h_insert{$insert});
						$nok = &TRUE;
						last;
						#last;
					}
				}
				$min = &Min($min,$#a_+1);
			}
			#delete ( $h_insert{$insert} ) if ( $min > 1 );
			if ($min == 1 )
			{
				$h_insert_ok{$insert}=1;
			}
		}
	}
	$self->_Print("OneSpecificEnd #insert post: ".scalar(keys(%h_insert))."\n");
	$self->_RemoveFromListOfInsert(\%h_insert_ok,$pattern);
	$self->_Print("Filter: OneSpecificEnd done\n");
#	my $total_size_after = total_size($rh_hspds);
#	$self->_Print("Filter: OneSpecificEnd mem_size before=$total_size_before, after=$total_size_after\n");
}

sub NoSpe
{
my($self) = @_;

	$self->_Print("Filter: NoSpe starting\n");
	my $pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	my $minlen  = $self->_FastOneGet('_nospeminlen');
	my $minpci  = $self->_FastOneGet('_nospeminpci');
	$self->_Print("Filter: target/nospeminlen=$minlen\n");
	$self->_Print("Filter: target/nospeminpci=$minpci\n");
	if ( ! defined($minpci) || ! defined($minlen) || $minpci eq '' || $minlen eq '' )
	{
		Carp::croak("nospeminlen and nospeminpci must be specified for lib ".$self->_FastOneGet('_target')."\n");
	}
	my $rh_hspds = $self->{_rh_hspds};
#	my $total_size_before = total_size($rh_hspds);
	my %h_insert=();
	my %h_insert_ok=();
	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($query) =$key =~ /query=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		$h_insert{$insert}{$ext} .= $rh_hspds->{$key}; # c'est inutile de rajouter ."##" car toutes les lignes finissent par ##
	}
	$self->_Print("NoSpe #inserts pre: ".keys(%h_insert)."\n");
	my %h_deja_vu = ();
	foreach my $insert(keys(%h_insert))
	{
		my @a_ends = keys(%{$h_insert{$insert}});
		next if ( $#a_ends != 1 ); # only one has been mapped, skipped

		my @a_1 = split(/##/,$h_insert{$insert}{$a_ends[0]});
		my @a_2 = split(/##/,$h_insert{$insert}{$a_ends[1]});

		# a la fin il faut que les deux soient definis, sinon je vire l'insert (une des deux ends n'aura pas passe le nouveau seuil)
		my $new_1 = undef;
		my $new_2 = undef;

		my $nb_1 = scalar(@a_1);
		if ( $nb_1 > 1 ) #  je ne teste les nouveaux seuils qu'en cas d'ambiguite de mapping
		{
			for(my $i=0;$i<$nb_1;$i++)
			{
				my %h_hsp = split(/[\s=]/,$a_1[$i]);
				if ( $h_hsp{len} >= $minlen && $h_hsp{pci} >= $minpci )
				{
					$new_1 = 'OK';
				}
				else
				{
					my $key = "query=$h_hsp{query} target=$h_hsp{target}";
					delete($rh_hspds->{$key});
				}
			}
		}
		else
		{
			$new_1 = 'OK' ;
		}
		if ( defined($new_1) )
		{
			my $nb_2 = scalar(@a_2);
			if ( $nb_2 > 1 )
			{
				for(my $i=0;$i<$nb_2;$i++)
				{
					my %h_hsp = split(/[\s=]/,$a_2[$i]);
					if ( $h_hsp{len} >= $minlen && $h_hsp{pci} >= $minpci )
					{
						$new_2 = 'OK';
					}
					else
					{
						my $key = "query=$h_hsp{query} target=$h_hsp{target}";
						delete($rh_hspds->{$key});
					}
				}
			}
			else
			{
				$new_2 = 'OK';
			}
		}
		if ( defined($new_1) && defined($new_2) )
		{
			$h_insert_ok{$insert}=1;
		}
	}
	$self->_RemoveFromListOfInsert(\%h_insert_ok,$pattern);
	$self->_Print("NoSpe #insert post: ".scalar(keys(%h_insert_ok))."\n");
	$self->_Print("Filter: NoSpe done\n");
#	my $total_size_after = total_size($rh_hspds);
#	$self->_Print("Filter: NoSpe mem_size before=$total_size_before, after=$total_size_after\n");
}


# ne garde que les paires ou les 2 extremites ont ete mappes sur 2 query distinctes

sub LinkByPE
{
my($self) = @_;

	$self->_Print("Filter: LinkByPE starting\n");
	my $pattern = $self->_FastOneGet('_target')->_FastOneGet('_pattern');
	$self->_Print("Filter: target/pattern=$pattern\n");
	my $rh_hspds = $self->{_rh_hspds};
	my %h_insert=();
	my $nboneend=0;
	my $nbsameseq=0;
	# identify paired ends
	foreach my $key (keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($query) =$key =~ /query=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		$h_insert{$insert}{$ext} .= "$query " if ( $h_insert{$insert}{$ext} !~ /$query/ );
	}
	$self->_Print("LinkByPE #insert pre: ".scalar(keys(%h_insert))."\n");
	my @a_ =();
	my $nbentries=0;
	foreach my $insert(keys(%h_insert))
	{
		my @a_ends = keys(%{$h_insert{$insert}});
		if ( $#a_ends != 1 ) # only one has been mapped
		{
			$nboneend++;
			delete($h_insert{$insert});
		}
		else
		{
			my %h_query = ();
			foreach my $end (@a_ends)
			{
				@a_ = split(' ',$h_insert{$insert}{$end});
				foreach my $q (@a_)
				{
					$h_query{$q}++;	
					if ( $h_query{$q} == 2 )	# 2 hits de la meme extremite sur la meme query ou les deux extremites sur la meme query
					{
						$nbsameseq++;
						delete($h_insert{$insert});
						last;
					}
				}
			}
		}
		$nbentries++ if ( exists($h_insert{$insert}) )
	}
	$self->_RemoveFromListOfInsert(\%h_insert,$pattern);
	$self->_Print("LinkByPE #insert post: ".scalar(keys(%h_insert))."\n");
	# remove not relevant hsps (one end mapped or both ends on the same query)
	$self->_Print("one-end deleted: $nboneend\n");
	$self->_Print("two-ends deleted (same query): $nbsameseq\n");
	$self->_Print("Filter: LinkByPE done\n");
	return $nbentries;
}

sub _Print
{
my($self,$msg)=@_;

	print STDERR $msg if (  $self->_FastOneGet('_verbose') );
}

sub _RemoveFromListOfInsert
{
my($self,$rh_insert,$pattern) = @_;

	my $rh_hspds = $self->{_rh_hspds};
	my @a_keys=keys(%{$rh_hspds});
	my ($target,$insert,$ext)=();
	for(my $i=0;$i<scalar(@a_keys);$i++)
	{
		($target)=$a_keys[$i] =~ /target=(\S+)/;
		($insert,$ext) = $target =~ /$pattern/;
		if ( !exists($$rh_insert{$insert}) )
		{
			delete($rh_hspds->{$a_keys[$i]});
		}
	}
}


1;
