#
# $Id: ArrayRef.pm 232 2013-12-27 17:22:42Z gouzy $
#

package HSPSet::ArrayRef;

use HSPSet;
use base qw( HSPSet );

use General;
use GeneralBioinfo;
use strict;
use warnings;

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 232 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	$self->SUPER::_Init(%h_param);
}

=head2 procedure Load

 Title        : Load
 Usage        : $o_hspds->Load;

=cut

sub Load 
{
my ($self,$ra_input,%h_param) = @_;

	my %h_min = ();
	my %h_max = ();

	$self->InitMinMax(\%h_min,\%h_max,'pci','len');

	my %h_hsp=();
	my $query_col = $self->_FastOneGet('_query_col');
	my $bestperquery  =  ( defined($h_param{onlybestperquery}) && $h_param{onlybestperquery} ) ? &TRUE: &FALSE;
	my $chain =          ( defined($h_param{chain} ) && $h_param{chain} ) ? &TRUE: &FALSE;
	my $splicedtarget = ( defined($h_param{spliced_target}   ) && $h_param{spliced_target} ) ? &TRUE: &FALSE;
	my ($cpt_in,$cpt_out)=(0,0);
	for(my $i=0;$i<scalar(@$ra_input);$i++)
	{	
		my $l=$$ra_input[$i];
		chomp($l);
		&ParseM8Line(\$l,$query_col,\%h_hsp);
		$cpt_in++;
		my $skip = 0;
		foreach my $criterium ('pci','len')
		{
			if ( $h_hsp{$criterium} < $h_min{$criterium} || $h_hsp{$criterium} > $h_max{$criterium} )
			{
				$skip = 1;
				last;
			}
		}
		next if ( $skip );
		my $val = $self->GetEntry(\%h_hsp);
		if ( $bestperquery )
		{
			my $ret = $self->AddIfBest("query=$h_hsp{query} target=$h_hsp{target}",$val,$h_hsp{len}*$h_hsp{pci});
			$cpt_out++ if ( $ret != 2 );
		}
		else
		{
			if ( $chain )
			{
				# if current hit is the best score, it replaces the previous one
				# otherwise, when it is compatible with the previous hit, it is "merged" with it (as its pci) 
				my $ret = $self->MergeWithOrReplaceBest("query=$h_hsp{query} target=$h_hsp{target}",$val,$h_hsp{len}*$h_hsp{pci},$splicedtarget);
				$cpt_out++ if ( $ret == 0 );
			}
			else
			{
				$self->Add("query=$h_hsp{query} target=$h_hsp{target}",$val);
				$cpt_out++;
			}
		}
	}
	# a supprimer
	$self->_Set('count',$cpt_out);
	$self->_Print("HSP in: $cpt_in\nHSP out: $cpt_out\n");
}

sub DumpArrayRef
{
my($self)=@_;

	my $querycol = $self->_FastOneGet('_query_col');
	my $rh_hspds = $self->{_rh_hspds};
	my %h_hsp=();
	my @a_hsps=();
	foreach my $key (sort keys(%{$rh_hspds}))
	{
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			%h_hsp =();
			%h_hsp = split(/[\s=]/,$entry);
			if ( $querycol == 2 )
			{
				my ($qstart,$qend) = ($h_hsp{strand} eq '+' ) ? ($h_hsp{qstart},$h_hsp{qend}) : ($h_hsp{qend},$h_hsp{qstart});
				push(@a_hsps,"$h_hsp{target}\t$h_hsp{query}\t$h_hsp{pci}\t$h_hsp{len}\t$h_hsp{mm}\t$h_hsp{gap}\t$h_hsp{tstart}\t$h_hsp{tend}\t$qstart\t$qend\t$h_hsp{'e-value'}\t$h_hsp{score}\n");
			}
			else
			{
				my ($tstart,$tend) = ($h_hsp{strand} eq '+' ) ? ($h_hsp{tstart},$h_hsp{tend}) : ($h_hsp{tend},$h_hsp{tstart});
				push(@a_hsps,"$h_hsp{query}\t$h_hsp{target}\t$h_hsp{pci}\t$h_hsp{len}\t$h_hsp{mm}\t$h_hsp{gap}\t$h_hsp{qstart}\t$h_hsp{qend}\t$tstart\t$tend\t$h_hsp{'e-value'}\t$h_hsp{score}\n");
			}
		}
	}
	print STDERR @a_hsps;
	return \@a_hsps;
}

1;
