#
# $Id$
#

package HSPSet::RMAP;

use HSPSet;
use base qw( HSPSet );

use General;
use GeneralBioinfo;
use strict;
use warnings;
use HSPSet::ArrayRef;
use Devel::Size qw(size total_size);
use Data::Dumper;

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 440 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	$self->SUPER::_Init(%h_param);
}


=head2 procedure Load

 Title        : Load
 Usage        : $o_hspds->Load;

=cut

sub Load 
{
my ($self,%h_param) = @_;

	my %h_min = ();
	my %h_max = ();

	$self->InitMinMax(\%h_min,\%h_max,'density','score','pcs','pcq');

	my %h_hsp=();
	my $filein = $self->_FastOneGet('_infile');
	$filein =~ s/\s//g;
	my @a_filein = split(',',$filein);	# infile=file1 ou infile=file1,file2
	$self->_Print("\n======================================\nLoading $filein starting\n");
	my ($cpt_in,$cpt_out)=(0,0);
	foreach my $fin (@a_filein)
	{
		my $f_inf = &GetStreamIn($fin);
		while (my $l = <$f_inf>)
		{	
			chomp($l);
			next if ( $l =~ /^ref/ );
			&ParseRmapLine(\$l,\%h_hsp);
			$cpt_in++;
			my $skip = 0;
			foreach my $criterium ('density','score','pcs','pcq')
			{
				if ( $h_hsp{$criterium} < $h_min{$criterium} || $h_hsp{$criterium} > $h_max{$criterium} )
				{
					$skip = 1;
					last;
				}
			}
			next if ( $skip );
			my $val = $self->GetEntry(\%h_hsp);
			my $ret = $self->AddIfBest("query=$h_hsp{query} target=$h_hsp{target}",$val,$h_hsp{score});
			$cpt_out++ if ( $ret != 2 );
		}
		$f_inf->close;
	}
	$self->_Set('count',$cpt_out);
	$self->_Print("HSP in: $cpt_in\nHSP out: $cpt_out\n");
	$self->_Print("Loading $filein done\n");
	return $h_param{filters};
}

#ref_id  ref_slen ref_sstart ref_send  ref_ntlen  ref_ntstart ref_ntend  ref_strand   tgt_id  tgt_slen  tgt_sstart  tgt_send  tgt_ntlen   tgt_ntstart  tgt_ntend  aln_len aln_score  aln_density  ref_span tgt_span
#0       1        2          3         4          5           6          7            8       9         10          11        12          13           14         15      16         17           18       19
#
#OcIN23-M4-r1c043        885     228     270     5861858 1417957 1634051 -       exp_refineFinal1_contig1000_r   58      1       40      356977  1       253936  43      12.39   0.2881  3.69    71.14
#OcIN23-M4-r1c638        63      11      63      297828  85033   289565  +       exp_refineFinal1_contig1000_r   58      15      56      356977  128772  335485  53      26.63   0.5025  68.67   57.91

sub ParseRmapLine
{
   my ($r_line, $rh_result) = @_;

    @_ = split("\t", $$r_line);
	$$rh_result{'query'}  = $_[0];
	$$rh_result{'strand'} = $_[7];
	$$rh_result{'target'} = $_[8];
    $$rh_result{'score'}  = $_[16];
    $$rh_result{'density'}= $_[17];
	$$rh_result{'pcq'}    = $_[18];
	$$rh_result{'pcs'}    = $_[19];
	$$rh_result{'pci'}    = 0;
	$$rh_result{'mm'}    = 0;
	$$rh_result{'gap'}    = 0;
	$$rh_result{'e-value'}    = 1;
	($$rh_result{'qstart'}, $$rh_result{'qend'}) = ($_[5], $_[6]);
	$$rh_result{'len'}    = $_[6]-$_[5]+1;
	($$rh_result{'tstart'}, $$rh_result{'tend'}) = ($_[13], $_[14]);

    return;
}

1;
