#
# $Id: HSPSetDistrib.pm 165 2012-10-10 18:28:36Z gouzy $
#

package HSPSetDistrib;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;
use LinkSetDistrib;


# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 165 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ('infile' );
	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init();

	foreach my $param (keys(%h_param))
	{
		$self -> _Set ("_$param", $h_param{$param});
	}

	$self -> _Set ('_rh_hspds', {});
	my $o_linkds = New LinkSetDistrib();
	$self -> _Set ('_o_linkds', $o_linkds);
}

=head2 procedure Load

 Title        : Load
 Usage        : $o_hspds -> Load;

=cut

sub Load 
{
	my ($self) = @_;
	Carp::croak('abstract method' );
}


sub AssertParams
{
my ($self,$rh_param,$ra_expected) = @_;
	
	foreach my $param (@{$ra_expected})
	{
		Carp::croak("Fatal error in constructor: $param must be provided\n") if ( !defined($$rh_param{$param}) );
	}
}

sub Add
{
my($self,$key,$val) = @_;

	$self->{_rh_hspds}->{$key} .= "$val##";
}

sub Set
{
my($self,$key,$val) = @_;

	$self->{_rh_hspds}->{$key} = "$val##";
}


sub AddIfBest
{
my($self,$key,$val,$score) = @_;

	my $best = 0;
	#best=1 equal
	#best=2 >
	my $rh_hspds = $self->{_rh_hspds};
	if ( ! defined($rh_hspds->{$key}) )
	{
		$self->Add($key,$val);
		return 0;
	}
	foreach my $entry (split(/##/,$rh_hspds->{$key}))
	{
		my $prevscore = $self->GetGlobalScore($entry);
		if ($prevscore<$score)
		{
			$self->Set($key,$val);
			return 2;
		}
		elsif ( $prevscore == $score )
		{
			$self->Add($key,$val);
			return 1;
		}
	}
	return 0;
}


sub MergeWithOrReplaceBest
{
my($self,$key,$val,$score,$splicedtarget) = @_;

	my $best = 0;
	#best=1 equal
	#best=2 >
	my $rh_hspds = $self->{_rh_hspds};
	if ( ! defined($rh_hspds->{$key}) )
	{
		$self->Add($key,$val);
		return 0;
	}
	my %h_param = ();
	$h_param{overlap}       = $self->_FastOneGet('_overlap');
	if ( $splicedtarget) # genome vs transcript
	{
		$h_param{maxgenomespan} = $self->_FastOneGet('_maxgenomespan');
	}
	else
	{
		$h_param{maxdifflength} = $self->_FastOneGet('_maxdifflength');
	}
if ( 0 )
{
	{
		Carp::croak("key $key must be defined and not empty") if ( !defined($h_param{$key}) && $h_param{$key} eq '' );
	}
}

	foreach my $entry (split(/##/,$rh_hspds->{$key}))
	{
		my $prevscore = $self->GetGlobalScore($entry);
		if ($prevscore<$score)
		{
			$self->Set($key,$val);
			return 2;
		}
		elsif ( $prevscore == $score )
		{
			#"Egalite: je ne sais pas quoi faire! donc je ne fais rien\n";
			#$self->Add($key,$val);
			return 1;
		}
		else
		{
			# when hits are compatibles the newentry contains new coordinates on target and query sequence (MinMin,MaxMax)
			my ($merged,$newentry) = $self->MergeCompatibleHSPs($entry,$val,$splicedtarget,%h_param);
			if ( $merged )
			{
				$self->Set($key,$newentry);
				return 3;
			}
		}
	}
	return 0;
}


sub MergeCompatibleHSPs
{
my($self,$ent1,$ent2,$splicedtarget,%h_param)=@_;
my $merged   = 0;
my $newentry = 0;

	my %h_1 = split(/[\s=]/,$ent1);
	my %h_2 = split(/[\s=]/,$ent2);

	# brin different on ne fait rien
	return (0,'') if ( $h_1{strand} ne $h_2{strand} );

	my $ovl = $h_param{overlap};

	# meme brin mais large inclusion
	if ( $h_2{tstart} > ($h_1{tstart}+$ovl) &&  $h_2{tend} < ($h_1{tend}+$ovl) && $h_2{qstart} > ($h_1{qstart}+$ovl) &&  $h_2{qend} < ($h_1{qend}+$ovl) )
	{
		#print STDERR "LARGE :  $h_2{tstart} > ($h_1{tstart}+$ovl) &&  $h_2{tend} < ($h_1{tend}+$ovl) && $h_2{qstart} > ($h_1{qstart}+$ovl) &&  $h_2{qend} < ($h_1{qend}+$ovl)\n";
		return (0,'') ;
	}
	my %h_hsp = %h_1;
	if ( $splicedtarget)
	{
		$h_hsp{qstart} = &Min($h_1{qstart},$h_2{qstart});
		$h_hsp{qend}   = &Max($h_1{qend},$h_2{qend});
		return (0,'') if ( ($h_hsp{qend}-$h_hsp{qstart}) >  $h_param{maxgenomespan} );
		$h_hsp{tstart} = &Min($h_1{tstart},$h_2{tstart});
		$h_hsp{tend}   = &Max($h_1{tend},$h_2{tend});
	}
	else
	{
		# the length should be identical on both sequences .. but there are 'N' 
		# avant de merger il faut verifier que la distance est a peut pret la meme sur les deux sequences
		# pour les hits genomique je pourrais verifier la compatibilite mais je prefere le garder separe
		# pour pouvoir reutiliser plus facilement
		$h_hsp{qstart} = &Min($h_1{qstart},$h_2{qstart});
		$h_hsp{qend}   = &Max($h_1{qend},$h_2{qend});
		$h_hsp{tstart} = &Min($h_1{tstart},$h_2{tstart});
		$h_hsp{tend}   = &Max($h_1{tend},$h_2{tend});
		my $delta = abs(($h_hsp{qstart}-$h_hsp{qend}) - ($h_hsp{tstart}-$h_hsp{tend}));
		#print STDERR "$delta abs(($h_hsp{qstart}-$h_hsp{qend}) - ($h_hsp{tstart}-$h_hsp{tend})) $ent1 $ent2\n";
		if ( $delta > $h_param{maxdifflength} )
		{
			return (0,'') 
		}
	}
	$h_hsp{len}    = $h_hsp{tend}-$h_hsp{tstart}+1;
	my $l = ( $h_hsp{len} > ( $h_2{len} + $h_1{len} ) ) ? ( $h_2{len} + $h_1{len} ) : $h_hsp{len};
	$h_hsp{pci}    = sprintf("%.2f",&Min(($h_1{pci}*$h_1{len}+$h_2{pci}*$h_2{len})/$l,100));
	# 20120616 $h_hsp{score}  = $h_hsp{pci};
	$h_hsp{score}  = int($l*2*$h_hsp{pci}/100); # pour blastn il faut ~ multiplier par 2 pour approximer
	# overestimated
	$h_hsp{gap}    = $h_1{gap}+$h_2{gap};
	$h_hsp{mm}     = $h_1{mm}+$h_2{mm};
	my $val = $self->GetEntry(\%h_hsp);
	return (1,$val);
	
}

sub GetGlobalScore
{
my($self,$entry) = @_;

	my ($len) = $entry =~ /len=(\d+)/;
	my ($pci) = $entry =~ /pci=(\S+)/;
	return sprintf("%.02f",$len*$pci);# pour eviter les problemes d'arrondis lorsque je cherche le meilleur
}


sub Get
{
my($self,$key) = @_;

	return $self->{_rh_hspds}->{$key};
}

sub GetHsps
{
my($self) = @_;

	return keys(%{$self->{_rh_hspds}});
}


sub DumpM8
{
my($self,$f_stream)=@_;

	my $querycol = $self->_FastOneGet('_query_col');
	my $rh_hspds = $self->{_rh_hspds};
	my %h_hsp=();
	my $refq = $self->_FastOneGet('_query');
	foreach my $key (sort keys(%{$rh_hspds}))
	{
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			%h_hsp =();
			%h_hsp = split(/[\s=]/,$entry);
			#my @a_fields = split(/\t/,$entry);	# query=val\ttarget=val\t...
			#foreach my $field (@a_fields)
			#{
			#	my @a_keyval = split('=',$field);
			#	$h_hsp{$a_keyval[0]} = $a_keyval[1];
			#}
			if ( $querycol == 2 )
			{
				my ($qstart,$qend) = ($h_hsp{strand} eq '+' ) ? ($h_hsp{qstart},$h_hsp{qend}) : ($h_hsp{qend},$h_hsp{qstart});
				print $f_stream "$h_hsp{target}\t$h_hsp{query}\t$h_hsp{pci}\t$h_hsp{len}\t$h_hsp{mm}\t$h_hsp{gap}\t$h_hsp{tstart}\t$h_hsp{tend}\t$qstart\t$qend\t$h_hsp{'e-value'}\t$h_hsp{score}\n";
			}
			else
			{
				my ($tstart,$tend) = ($h_hsp{strand} eq '+' ) ? ($h_hsp{tstart},$h_hsp{tend}) : ($h_hsp{tend},$h_hsp{tstart});
				print $f_stream "$h_hsp{query}\t$h_hsp{target}\t$h_hsp{pci}\t$h_hsp{len}\t$h_hsp{mm}\t$h_hsp{gap}\t$h_hsp{qstart}\t$h_hsp{qend}\t$tstart\t$tend\t$h_hsp{'e-value'}\t$h_hsp{score}\n";
			}
		}
	}
}

sub PrintInfo
{
my($self) = @_;

	my $refq = $self->_FastOneGet('_query');
	my $reft = $self->_FastOneGet('_target');

	my $rh_hspds = $self->{_rh_hspds};
	my $nb_keys = keys(%{$rh_hspds});
	$self->_Print("Number of query-target: $nb_keys\n");
}

sub GetEntry
{
my ($self,$rh_hsp) = @_;

	my $val = '';
	foreach my $k (sort keys(%$rh_hsp))
	{
		$val .= "$k=$$rh_hsp{$k}\t";
	}
	return $val;
}

sub InitMinMax
{
my($self,$rh_min,$rh_max,@a_criteria)=@_;

	foreach my $criterium (@a_criteria)
	{
		$$rh_min{$criterium} = ( $self->_FastOneGet("_min$criterium") ) ? $self->_FastOneGet("_min$criterium") : 0;
		$$rh_max{$criterium} = ( $self->_FastOneGet("_max$criterium") ) ? $self->_FastOneGet("_max$criterium") : &MAXINT;
	}
}

#
#============= Filters =================
#

# on conserve tous les meilleurs scores

# je n'analyse pas les entries, c'est un QueryCount

sub QueryCount
{
my($self) = @_;

	my %h_min=();
	my %h_max=();
	$self->InitMinMax(\%h_min,\%h_max,'count');

	$self->_Print("Filter: QueryCount starting\n");
	$self->_Print("\tmincount=$h_min{count}, maxcount=$h_max{count}\n");
	
	my %h_target = ();
	
	my $rh_hspds = $self->{_rh_hspds};
	# first pass, count
	foreach my $key (sort keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		$h_target{$target}++;
	}

	# second pass, removal of low/high counts
	foreach my $key (sort keys(%{$rh_hspds}))
	{	
		my ($target)=$key =~ /target=(\S+)/;
		if( ($h_target{$target} < $h_min{'count'}) || ($h_target{$target} > $h_max{'count'}) )
		{
			delete($rh_hspds->{$key});
		}	
	}
	$self->_Print("Filter: QueryCount done\n");
}

#
# ne garde que sequences qui permettent de faire le lien entre deux query
# Prerequis: il faut d'abord passer QueryCount avec un mincount >= 2
# les hits doivent etre compatibles sur la target
#

sub LinkBySeq
{
my($self,$r_tseq) = @_;

	$self->_Print("Filter: LinkBySeq starting\n");
	my $ovl = $self->_FastOneGet('_overlap');
	my $rh_hspds = $self->{_rh_hspds};
	my %h_target=();
	# je collecte les donnees par target
	foreach my $key (sort keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		foreach my $entry (split(/##/,$rh_hspds->{$key}))
		{
			$h_target{$target} .= "$entry##";
		}
	}
	my %h_target2query=();
	# j'analyse les paires de hits pour voir s'ils sont compatibles sur la target
	foreach my $target (keys(%h_target))
	{
		my @a_   = split(/##/,$h_target{$target});
		my @a_ok = split('',0 x ($#a_+1));
		for (my $i=0; $i<$#a_;$i++)
		{
			my %h_i = split(/[\s=]/,$a_[$i]);
			for (my $j=$i+1; $j<=$#a_;$j++)
			{
				my %h_j = split(/[\s=]/,$a_[$j]);
				if ( $h_i{tstart} < $h_j{tstart} )
				{
					if ( $h_i{tend} < ($h_j{tstart}+$ovl) )
					{	
						$a_ok[$i]=1;
						$a_ok[$j]=1;
					}
				}
				else
				{
					if ( $h_j{tend} < ($h_i{tstart}+$ovl) )
					{	
						$a_ok[$i]=1;
						$a_ok[$j]=1;
					}
				}
			}
		}	
		for (my $i=0;$i<=$#a_;$i++)
		{
			my ($query) = $a_[$i] =~ /query=(\S+)/;
			$h_target2query{$target} .= "query=$query " if ($a_ok[$i] == 1 );
		}
	}
	foreach my $key (sort keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($query) =$key =~ /query=(\S+)/;
		if ( !defined($h_target2query{$target}) || $h_target2query{$target} !~ /$query\s/ )
		{
			delete($rh_hspds->{$key});
		}
	}
	$self->_Print("Filter: LinkBySeq done\n");
}

sub _Print
{
my($self,$msg)=@_;

	print STDERR $msg if (  $self->_FastOneGet('_verbose') );
}


1;
