#
# $Id: M8.pm 165 2012-10-10 18:28:36Z gouzy $
#

package HSPSetDistrib::M8;

use HSPSetDistrib;
use base qw( HSPSet );

use General;
use GeneralBioinfo;
use strict;
use warnings;

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 165 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ('query_col');
	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init(%h_param);
}

=head2 procedure Load

 Title        : Load
 Usage        : $o_hspds->Load;

=cut

sub Load 
{
my ($self,%h_param) = @_;

	my %h_min = ();
	my %h_max = ();

	$self->InitMinMax(\%h_min,\%h_max,'pci','len');

	my %h_hsp=();
	my $filein = $self->_FastOneGet('_infile');
	$self->_Print("\n======================================\nLoading $filein starting\n");
	my $f_inf = &GetStreamIn($filein);
	my $query_col = $self->_FastOneGet('_query_col');
	my $bestperquery  =  ( defined($h_param{onlybestperquery}) && $h_param{onlybestperquery} ) ? &TRUE: &FALSE;
	my $chain =          ( defined($h_param{chain} ) && $h_param{chain} ) ? &TRUE: &FALSE;
	my $splicedtarget = ( defined($h_param{spliced_target}   ) && $h_param{spliced_target} ) ? &TRUE: &FALSE;
	my ($cpt_in,$cpt_out)=(0,0);
	while (my $l = <$f_inf>)
	{	
		chomp($l);
		&ParseM8Line(\$l,$query_col,\%h_hsp);
		$cpt_in++;
		my $skip = 0;
		foreach my $criterium ('pci','len')
		{
			if ( $h_hsp{$criterium} < $h_min{$criterium} || $h_hsp{$criterium} > $h_max{$criterium} )
			{
				$skip = 1;
				last;
			}
		}
		next if ( $skip );
		my $val = $self->GetEntry(\%h_hsp);
		if ( $bestperquery )
		{
			my $ret = $self->AddIfBest("query=$h_hsp{query} target=$h_hsp{target}",$val,$h_hsp{len}*$h_hsp{pci});
			$cpt_out++ if ( $ret != 2 );
		}
		else
		{
			if ( $chain )
			{
				# if current hit is the best score, it replaces the previous one
				# otherwise, when it is compatible with the previous hit, it is "merged" with it (as its pci) 
				my $ret = $self->MergeWithOrReplaceBest("query=$h_hsp{query} target=$h_hsp{target}",$val,$h_hsp{len}*$h_hsp{pci},$splicedtarget);
				$cpt_out++ if ( $ret == 0 );
			}
			else
			{
				$self->Add("query=$h_hsp{query} target=$h_hsp{target}",$val);
				$cpt_out++;
			}
		}
	}
	$f_inf->close;
	$self->_Set('count',$cpt_out);
	$self->_Print("HSP in: $cpt_in\nHSP out: $cpt_out\n");
	$self->_Print("Loading $filein done\n");
}


1;
