#
# $Id: LinkSetDistrib.pm 165 2012-10-10 18:28:36Z gouzy $
#

package LinkSetDistrib;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

my %H_SCAFF2EXT    = ( "p p" => "3 5", "p m" => "3 3", "m p" => "5 5", "m m" => "5 3" );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 165 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ();
	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init();

	foreach my $param (keys(%h_param))
	{
		$self -> _Set ("_$param", $h_param{$param});
	}

	$self -> _Set ('_rh_linkds', {});
	$self -> _Set ('_rh_stats', {});
	$self -> _Set ('_nb_links', 0);
}

sub AssertParams
{
my ($self,$rh_param,$ra_expected) = @_;
	
	foreach my $param (@{$ra_expected})
	{
		if ( !defined($$rh_param{$param}) )
		{
			print Dumper(%$rh_param);
			Carp::croak("Fatal error in constructor: $param must be provided\n") 
		}
	}
}

sub IsEmpty
{
my($self)=@_;

	return 1 if ( ! $self -> _FastOneGet ('_nb_links') );
	return 0;
}

sub Reset
{
my($self)=@_;

	$self -> _Set ('_rh_linkds', {});
	$self -> _Set ('_nb_links',0);
}

sub BuildKey
{
my($self,$id1,$ext1,$id2,$ext2)=@_;

	if ($id1 lt $id2)
	{
		return "src=$id1 tgt=$id2";
	}
	else
	{
		return "src=$id2 tgt=$id1";
	}
}

sub Del
{
my($self,$key)=@_;

	if ( defined($self->{'_rh_linkds'}->{$key}) )
	{
		delete ($self->{'_rh_linkds'}->{$key});
		$self->{'_nb_linkds'}--;
	}
}

sub Del2
{
my($self,$id1,$id2)=@_;

	my ($src,$tgt)=sort($id1,$id2);
	my $key = "src=$src tgt=$tgt";
	if ( defined($self->{'_rh_linkds'}->{$key}) )
	{
		delete ($self->{'_rh_linkds'}->{$key});
		$self->{'_nb_linkds'}--;
	}
}

sub Add
{
my($self,$key,$val) = @_;

	if ( ! defined($self->{'_rh_linkds'}->{$key}) )
	{
		$self->{'_rh_linkds'}->{$key} = $val;
		$self->{'_nb_linkds'}++;
	}
	else
	{
		$self->{'_rh_linkds'}->{$key} .= "##$val";
	}
}

sub AddKV
{
my($self,$key,$rh_val) = @_;

	my $str = '';
	foreach my $k (sort keys(%$rh_val))
	{
		$str .= "$k=$$rh_val{$k} ";
	}
	chop($str);
	if ( ! defined($self->{'_rh_linkds'}->{$key}) )
	{
		$self->{'_rh_linkds'}->{$key} = $str;
		$self->{'_nb_linkds'}++;
	}
	else
	{
		$self->{'_rh_linkds'}->{$key} .= "##$str";
	}
}


sub Set
{
my($self,$key,$val) = @_;

	$self->{'_nb_linkds'}++ if ( ! defined($self->{'_rh_linkds'}->{$key}) );
	$self->{'_rh_linkds'}->{$key} = $val;
}

sub SetKV
{
my($self,$key,$rh_val)=@_;

	$self->{'_nb_linkds'}++ if ( ! defined($self->{'_rh_linkds'}->{$key}) );
	my $str = '';
	foreach my $k (sort keys(%$rh_val))
	{
		$str .= "$k=$$rh_val{$k} ";
	}
	chop($str);
	$self->{'_rh_linkds'}->{$key}=$str;
}

sub SetKV2
{
my($self,$id1,$id2,$rh_val)=@_;

	my ($src,$tgt)=sort($id1,$id2);
	my $key = "src=$src tgt=$tgt";
	$self->{'_nb_linkds'}++ if ( ! defined($self->{'_rh_linkds'}->{$key}) );
	$self->SetKV($key,$rh_val);
}

# returns, key-value hash
# ATTENTION: je regarde dans tous les liens charges pas dans les filtres

sub GetKV
{
my($self,$key,$rh_val)=@_;

	if (defined($self->{'_rh_linkds'}->{$key}))
	{
		%{$rh_val} = split(/[\s=]+/,$self->{'_rh_linkds'}->{$key});
		# implicit return
	}
	else
	{
		return &FALSE;
	}
}

# ATTENTION: je regarde dans tous les liens charges pas dans les filtres

sub GetKV2
{
my($self,$id1,$id2,$rh_val)=@_;

	my ($src,$tgt)=sort($id1,$id2);
	my $key = "src=$src tgt=$tgt";
	return $self->GetKV($key,$rh_val);
}

# ATTENTION: je regarde dans tous les liens charges pas dans les filtres

sub GetKV2Oriented
{
my($self,$id1,$ext1,$id2,$ext2,$rh_val)=@_;

	my $key = '';
	my ($srcext,$tgtext);
	if ( $id1 lt $id2 )
	{
		$key = "src=$id1 tgt=$id2";
		($srcext,$tgtext) = ($ext1,$ext2);
	}
	else
	{
		$key = "src=$id2 tgt=$id1";
		($srcext,$tgtext) = ($ext2,$ext1);
	}
	return &FALSE if ( ! $self->GetKV($key,$rh_val) );

	if ( $$rh_val{srcext} != $srcext || $$rh_val{tgtext} != $tgtext )
	{
		#print STDERR "FALSE: $key $$rh_val{srcext} != $srcext || $$rh_val{tgtext} != $tgtext \n";
		return &FALSE;
	}
	return %{$rh_val};
}

# ATTENTION: je regarde dans tous les liens charges pas dans les filtres

sub GetKV2OrientedContigs
{
my($self,$id1,$strand1,$id2,$strand2,$rh_val)=@_;

	my ($ext1,$ext2) = split(' ',$H_SCAFF2EXT{"$strand1 $strand2"});
	
	#print STDERR "$id1,$strand1,$id2,$strand2 => ($ext1,$ext2)\n";

	return $self->GetKV2Oriented($id1,$ext1,$id2,$ext2,$rh_val);
}

# idem mais tri numerique (pour les idscaff)

sub GetKV2N
{
my($self,$id1,$id2,$rh_val)=@_;

	my ($src,$tgt)=sort { $a<=>$b }($id1,$id2);
	my $key = "src=$src tgt=$tgt";
	return $self->GetKV($key,$rh_val);
}

sub GetLinks
{
my($self) = @_;

	return keys(%{$self->{_rh_linkds}});
}


sub Get
{
my($self,$key) = @_;

	return $self->{'_rh_linkds'}->{$key};
}


sub ComputeStats
{
my ($self,$rh_seq,%h_param) = @_;

	my @a_counts = ('weak','hardlink','insert_size','used','srcext','tgtext');
	my $outprefix = ( defined($h_param{statfile}) ) ? $h_param{statfile} : "LynxStats.$$.".time;
	my @a_links = ();
	$self->Filter(\@a_links,%h_param);
	my $f_out = &GetStreamOut("$outprefix.log");
	my %h_stats = ();
	my %h_ctg   = ();
	my @a_ratiospan = ();
	my @a_span = ();
	my @a_count = ();
	my @a_gaplen_max = ();
	my @a_gaplen_min = ();
	my $spansupinsert=0;
	my $nb_links = 0;
	my $max_insert_size=0;
	for(my $i=0;$i<=$#a_links;$i++)
	{
		my $link=$a_links[$i];
		my %h_key=split(/[=\s]/,$link);
		my $lk = $self->Get($link);
		$lk=~s/##$//;
		my @a_lk = split(/#+/,$lk);
		$h_stats{nb_orientation}{scalar(@a_lk)}++;
		foreach my $lk (@a_lk)
		{
			my %h_val=split(/[=\s]+/,$lk);
			$nb_links++;
			foreach my $tag (@a_counts)
			{
				next if ( ! defined($h_val{$tag}) );
				$h_stats{$tag}{$h_val{$tag}}++;
			}
			$h_ctg{$h_key{src}}{len}=$h_val{srclen};
			$h_ctg{$h_key{tgt}}{len}=$h_val{tgtlen};
			$h_ctg{$h_key{src}}{$h_val{srcext}}++;
			$h_ctg{$h_key{src}}{"len$h_val{srcext}"}+=$h_val{tgtlen};
			$h_ctg{$h_key{tgt}}{"len$h_val{tgtext}"}+=$h_val{srclen};
			$h_ctg{$h_key{tgt}}{$h_val{tgtext}}++;
			my $ratiospan = ( $h_val{srcspan} > $h_val{tgtspan} ) ? ($h_val{srcspan}/$h_val{tgtspan}) : ($h_val{tgtspan}/$h_val{srcspan});
			$spansupinsert++ if( $h_val{srcspan} > $h_val{insert_size} || $h_val{tgtspan} > $h_val{insert_size} );
			push(@a_count,$h_val{count});
			push(@a_ratiospan,$ratiospan);
			push(@a_span,$h_val{srcspan},$h_val{tgtspan});
			push(@a_gaplen_max,$h_val{gaplen_max});
			push(@a_gaplen_min,$h_val{gaplen_min});
			$max_insert_size=&Max($h_val{insert_size},$max_insert_size);
		}
	}
	my @a_class = (1,2,3,4,5,6,7,8,9,10);
	&Histo(\@a_ratiospan,\@a_class,&GetStreamOut("$outprefix.ratiospan.tsv"));
	@a_class = (100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 2000, 3000, 4000, 5000, 10000, 20000);
	&Histo(\@a_span,\@a_class,&GetStreamOut("$outprefix.span.tsv"));
	&Histo(\@a_gaplen_max,\@a_class,&GetStreamOut("$outprefix.gaplen_max.tsv"));
	&Histo(\@a_gaplen_min,\@a_class,&GetStreamOut("$outprefix.gaplen_min.tsv"));
	@a_class = (1,2,3,4,5,6,7,8,9,10,20,30,40,50,60,70,80,90,100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 2000, 3000, 4000, 5000, 10000, 20000);
	&Histo(\@a_count,\@a_class,&GetStreamOut("$outprefix.count.tsv"));
	my @a_ctg = sort keys(%{$rh_seq});
	my $f_unlinked=&GetStreamOut("$outprefix.unlinked.ctg.fa");
	my $f_linkedthreeonly=&GetStreamOut("$outprefix.linkedthreeonly.ctg.fa");
	my $f_linkedfiveonly=&GetStreamOut("$outprefix.linkedfiveonly.ctg.fa");
	my $nb_unlinked=0;
	my $len_unlinked=0;
	my @a_five_lk = ();
	my @a_three_lk = ();
	$max_insert_size *= 1.1;
	foreach my $ctg (@a_ctg)
	{	
		if ( defined($h_ctg{$ctg}) )
		{
			if ( defined($h_ctg{$ctg}{5}) && defined($h_ctg{$ctg}{3}) )
			{
				$h_stats{connected}{both}++;
				push(@a_five_lk,$h_ctg{$ctg}{5});
				push(@a_three_lk,$h_ctg{$ctg}{3});
				#print "$ctg  both $h_ctg{$ctg}{5} $h_ctg{$ctg}{3}\n" if ( $h_ctg{$ctg}{5} > 10 || $h_ctg{$ctg}{3} > 10 );
				#print "$ctg  lenboth $h_ctg{$ctg}{len5} $h_ctg{$ctg}{len3}\n" if ( $h_ctg{$ctg}{len5} > $max_insert_size || $h_ctg{$ctg}{len3} > $max_insert_size );
			}
			elsif( defined($h_ctg{$ctg}{5}) )
			{
				$h_stats{connected}{fiveonly}++;
				print $f_linkedfiveonly ">$ctg len=$$rh_seq{$ctg}{len}\n".&FormatSeq(\$$rh_seq{$ctg}{sequence})."\n";
				push(@a_five_lk,$h_ctg{$ctg}{5});
				#print "$ctg five $h_ctg{$ctg}{5}\n"       if ( $h_ctg{$ctg}{5} > 100 );
				#print "$ctg lenfive $h_ctg{$ctg}{len5}\n" if ( $h_ctg{$ctg}{len5} > $max_insert_size );
			}
			elsif( defined($h_ctg{$ctg}{3}) )
			{
				$h_stats{connected}{threeonly}++;
				print $f_linkedthreeonly ">$ctg len=$$rh_seq{$ctg}{len}\n".&FormatSeq(\$$rh_seq{$ctg}{sequence})."\n";
				push(@a_three_lk,$h_ctg{$ctg}{3});
				#print "$ctg three $h_ctg{$ctg}{3}\n"       if ( $h_ctg{$ctg}{3} > 100 );
				#print "$ctg lenthree $h_ctg{$ctg}{len3}\n" if ( $h_ctg{$ctg}{len3} > $max_insert_size );
			}
		}
		else
		{
			print $f_unlinked ">$ctg len=$$rh_seq{$ctg}{len}\n".&FormatSeq(\$$rh_seq{$ctg}{sequence})."\n";
			$len_unlinked+=$$rh_seq{$ctg}{len};
			$nb_unlinked++;
		}
	}
	$f_unlinked->close;
	&Histo(\@a_five_lk,\@a_class,&GetStreamOut("$outprefix.links5.tsv"));
	&Histo(\@a_three_lk,\@a_class,&GetStreamOut("$outprefix.links3.tsv"));
	print $f_out "Nb contigs: ".scalar(@a_ctg)."\n";
	print $f_out "Nb unlinked contigs: $nb_unlinked\n";
	print $f_out "Size represented in unlinked contigs: $len_unlinked\n";
	print $f_out "Nb links: ".scalar(@a_links)." mult=$nb_links\n";
	print $f_out "Nb links span > insert_size: $spansupinsert\n";
	foreach my $tag (sort keys(%h_stats))
	{	
		foreach my $k (keys(%{$h_stats{$tag}}))
		{
			print $f_out "$tag\t$k\t$h_stats{$tag}{$k}\n";
		}
	}
	$self -> _Set ('_rh_stats', \%h_ctg);
}

sub Dump
{
my($self,%h_param)=@_;

	if ( ! defined($h_param{dumpfile}) )
	{
		print STDERR "Warning: dumping links failed, dumpfile parameter undefined\n";
		return;
	}
	$self->_Print("Dumping links >$h_param{dumpfile}< starting\n");
	my $f_links = &GetStreamOut($h_param{dumpfile});
	my @a_links = $self->GetLinks();
	foreach my $link (@a_links)
	{
		my $linkdetails = $self->Get($link);
		print $f_links "$link\t$linkdetails\n";
	}
	$f_links->close;
	$self->_Print("Dumping links: done\n");
}


sub Load
{
my($self,%h_param)=@_;

	if ( ! defined($h_param{dumpfile}) )
	{
		print STDERR "Warning: loading links failed, dumpfile parameter undefined\n";
		return;
	}
	$self->_Print("Loading links >$h_param{dumpfile}< starting\n");
	my $f_links = &GetStreamIn($h_param{dumpfile});
	while(my $l=<$f_links>)
	{
		chomp($l);
		my ($link,$details) = split("\t",$l,2);
		$self->Set($link,$details);
	}
	$f_links->close;
	$self->_Print("Loading links: done\n");
}


sub Filter
{
my($self,$ra_filt_links,%h_param)=@_;

	my @a_links_raw = $self->GetLinks();
	@$ra_filt_links=();
	my $nb_ignored=0;
	my $nb_tot=0;
	foreach my $link (@a_links_raw)
	{
		$nb_tot++;
		my %h_val = ();
		next if ( !$self->GetKV($link,\%h_val) );
		my $ignored=&FALSE;
		foreach my $k (keys(%h_param))		# si on arrive avec un mincount=100, on vire tous les links a < 100
											# si on arrive avec un maxinsert_size=4000, on vire les insert_size > 4000
		{	
			if ( $k =~ /^min(\S+)/ )
			{
				$ignored = &TRUE if ( defined($h_val{$1}) && $h_val{$1} < $h_param{$k} );
			}
			elsif ( $k =~ /^max(\S+)/ )
			{
				$ignored = &TRUE if ( defined($h_val{$1}) && $h_val{$1} > $h_param{$k} );
			}
			#print STDERR "Ignored $k: $link, $linkdetails \n" if ($ignored);
			last if ($ignored);
		}
		if ( $ignored )
		{
			$nb_ignored++;
		}
		else
		{
			push(@$ra_filt_links,$link);
		}
	}
	$self->_Print("Filter: $nb_ignored links ignored / $nb_tot\n");
	return $nb_ignored;
}


sub __RemoveFromListOfInsert
{
my($rh_hspds,$rh_insert,$pattern) = @_;

	foreach my $key (sort keys(%{$rh_hspds}))
	{
		my ($target)=$key =~ /target=(\S+)/;
		my ($insert,$ext) = $target =~ /$pattern/;
		if ( !defined($$rh_insert{$insert}) )
		{
			delete($rh_hspds->{$key});
		}
	}
}

sub _UpdHSP
{
my($r_str,$hsp)=@_;

	if ( ! defined($r_str) || $$r_str eq '')
	{
		$$r_str = $hsp;
	}
	else
	{
		$$r_str .= "##$hsp";
	}
}


sub _Print
{
my($self,$msg)=@_;

	print STDERR "$msg";
}


sub PrintInfo
{
my($self) = @_;

}


1;
