#
# $Id: Loader.pm 233 2013-12-27 17:23:15Z gouzy $
#

package Loader;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;


# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 233 $ =~ /\d+/g); $r[0]};
}

use strict;
use warnings;

use General;
use GeneralBioinfo;
use ParamParser;
use HSPSet::M8;
use SequenceSet::Hash;
use Data::Dumper;

my $DEBUG = 1;

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;
	
	my @a_expected = ('cfg','outdir');
	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init();

	my @a_tmp = ();
	$self->_Set('_rh_o_hspds', {});
	$self->_Set('_rh_o_seqds', {});
	$self->_Set('_ra_querylist',\@a_tmp);

	my $o_param = New ParamParser($h_param{cfg});
	foreach my $param (keys(%h_param))
	{
		$self -> _Set ("_$param", $h_param{$param});
		$o_param->Set($param,$h_param{$param});
	}
	$o_param->SetUnlessDefined('outfmt','M8');
	$o_param->AssertAllowedValue('outfmt','M8','Links');
	my $outfmt = $o_param->Get('outfmt');

	if ( ! $o_param->IsDefined('hsp_list') )
	{
		print STDERR "Loader Warning: hsp_list undefined\n";
	}
	if ( ! $o_param->IsDefined('sequence_list') )
	{
		print STDERR "Loader Warning: sequence_list undefined\n";
	}
	$o_param->AssertDirExists('outdir');
	my $outdir = $o_param->Get('outdir');

	my @a_hsplist=split(' ',$o_param->Get('hsp_list'));
	my @a_seqlist=split(' ',$o_param->Get('sequence_list'));

	#
	# Load sequence datasets
	#
	my $rh_o_seqds=$self->GetSequenceSets();

	foreach my $seqset (@a_seqlist)
	{
		$o_param->SelectNameSpace("sequence_${seqset}_");
		my @a_opt = $o_param->GetKeys();
		my %h_param = ();
		foreach my $opt (@a_opt)
		{
			$h_param{$opt} = $o_param->Get($opt);
		}
		if ( $h_param{format} eq 'Fasta' )
		{
			$$rh_o_seqds{$seqset} = New SequenceSet::Hash(%h_param);
		}
		else
		{
			Carp::croak("Unknown format >$h_param{format}<\n");
		}
		$$rh_o_seqds{$seqset}->Load(%h_param);
		$$rh_o_seqds{$seqset}->PrintInfo();
	}
	$o_param->SelectNameSpace('');

	#
	# Load HSP datasets
	#
	my $rh_o_hspds=$self->GetHSPSets();
	my $ra_querylist=$self->GetQueryList();
	my %h_querylist=();
	my $nohsp = ( $o_param->IsDefined('nohsp') ) ? &TRUE : &FALSE;
	foreach my $hspset (@a_hsplist)
	{
		$o_param->SelectNameSpace("hsp_${hspset}_");
		my @a_opt = $o_param->GetKeys();
		my %h_param = ();
		foreach my $opt (@a_opt)
		{
			$h_param{$opt} = $o_param->Get($opt);
			if ( $opt eq 'query' || $opt eq 'target' )
			{
				if ( !defined($$rh_o_seqds{$h_param{$opt}}) )
				{
					Carp::croak("Sequence dataset >$h_param{$opt}< must be loaded before loading any related HSP dataset\n");
				}
				if ( $opt eq 'query' )
				{
					$h_querylist{$h_param{$opt}}=1;
				}
				$h_param{$opt} = $$rh_o_seqds{$h_param{$opt}};
			}
		}
		if ( $h_param{format} eq 'M8' )
		{
			$$rh_o_hspds{$hspset} = New HSPSet::M8(%h_param);
		}
		else
		{
			Carp::croak("Unknown format >$h_param{format}<\n");
		}
		$$rh_o_hspds{$hspset}->Load(%h_param) if ( ! $nohsp );
		$$rh_o_hspds{$hspset}->PrintInfo();
	}
	$o_param->SelectNameSpace('');
	@$ra_querylist=keys(%h_querylist);
}



sub GetHSPSets
{
my($self) = @_;

	return $self -> _FastOneGet ('_rh_o_hspds');
}



sub GetSequenceSets
{
my($self) = @_;

	return $self -> _FastOneGet ('_rh_o_seqds');
}

sub GetQueryList
{
my($self) = @_;

	return $self -> _FastOneGet ('_ra_querylist');
}


sub AssertParams
{
my ($self,$rh_param,$ra_expected) = @_;
	
	foreach my $param (@{$ra_expected})
	{
		Carp::croak("Fatal error in constructor: $param must be provided\n") if ( !defined($$rh_param{$param}) );
	}
}


