#
# $Id: PreGeneralBioinfo.pm 29 2011-04-13 06:08:15Z gouzy $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# Erika.Sallet@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package PreGeneralBioinfo;

BEGIN 
{
        use Exporter   ();
        use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
        $VERSION = do { my @r = (q$Rev: 29 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };

        @ISA         = qw(Exporter);
        @EXPORT      = qw(FastaStreamToHash);
        %EXPORT_TAGS = ( );     # ex. : TAG => [ qw!name1 name2! ],
        @EXPORT_OK   = qw(FastaStreamToHash);
}


use Carp;
use strict;

=head2 function FastaStreamToHash

  Title        : FastaStreamToHash
  Usage        : &FastaStreamToHash($fh_fastastream,\%h_seq);
  Prerequisite : sequences are clean
  Function     : Load a fasta stream in a hash
  Returns      : the number of sequences loaded
  Args         : $fh_fastastream: a file handle
               : \%h_seq: a reference to a hash
  Globals      : none

=cut

sub FastaStreamToHash
{
my($fh_,$rh_) = @_;

	return 0 if ( ! defined($fh_) ); 
	my $id='';
	my $cpt=0;
	while(my $l=<$fh_>)
	{
		chomp($l);
		if ( $l =~ /^>(\S+)(.*)/ )
		{
			$cpt++;
			$id = $1;
			if ( defined($$rh_{$id}{SEQ}) )
			{
				Carp::croak("Error: sequence $id already loaded\n");
			}
			my $hd = ( defined($2) ) ? $2 : "";
			$$rh_{$id}{HEADER}=$hd;
			
		}
		else
		{
			$$rh_{$id}{SEQ}.=$l;
		}
	}
	return $cpt;
}

1;
