#
# $Id: shared.pm 222 2013-12-27 17:10:16Z gouzy $
#

package RamBdbHash::shared;


use RamBdbHash;
use base qw(RamBdbHash);

use threads;
use threads::shared;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;
use BerkeleyDB;

use File::Temp;

sub BerkeleyDB::Hash::DESTROY
{
        ; #print STDERR "Destroy BerkeleyDB::Hash::DESTROY\n";
}

sub BerkeleyDB::Env::DESTROY
{
        ; #print STDERR "Destroy BerkeleyDB::Env::DESTROY\n";
}


# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 222 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ('lock_hash','lock_bdb');
	$self->AssertParams(\%h_param,\@a_expected);

	if ( ! is_shared($h_param{lock_hash}) || ! is_shared($h_param{lock_bdb}) )
	{
			Carp::croak("Error: lock_lock and lock_bdb must be shared variables\n");
	}

	$self->SUPER::_Init();

	foreach my $param (keys(%h_param))
	{
		$self -> _Set ("_$param", $h_param{$param});
	}

	my %h_hash:shared=();
	$self->_Set('_rh_hash', \%h_hash);
}

sub Reset
{
my($self)=@_;

	my $lock_bdb = $$self{'_lock_bdb'};
	{
		my $o_bdb = $$self{'_o_bdb'};
		lock($lock_bdb);
		my ($k, $v) = ("", "") ;
		my $cursor = $o_bdb->db_cursor() ;
		while ($cursor->c_get($k, $v, DB_NEXT) == 0)
		{
			$o_bdb->db_del($k);
		}
		undef $cursor ;
	}
	my $lock_hash = $$self{'_lock_hash'};
	{
		lock($lock_hash);
		my $rh_hash = $$self{'_rh_hash'};
		%{$rh_hash}=();
	}
}

# je regarde d'abord dans le hash puis dans la bdb

sub Get
{
my($self,$key) = @_;

	my $val=undef;
	{
		lock($$self{'_lock_hash'});
		$val = $self->{_rh_hash}->{$key};
	}
	if ( ! defined($val) )
	{
		my $o_bdb = $$self{'_o_bdb'};
		lock($$self{'_lock_bdb'});
		$o_bdb->db_get($key,$val);
	}
	return $val;
}

sub Exists
{
my($self,$key) = @_;

	my $exists;
	{
		lock($$self{'_lock_hash'});
		$exists = exists($self->{_rh_hash}->{$key});
	}
	if ( ! $exists )
	{
		my $o_bdb = $$self{'_o_bdb'};
		lock($$self{'_lock_bdb'});
		my $status = $o_bdb->db_exists($key);
		$status = ( $status eq '' ) ? 1 : 0; # c'est bizarre le status est '' lorsqu'il trouve
		return $status;
	}
	else
	{
		return 1;
	}
}


# si la cle du hash existe je l'utilise sinon bdb par defaut

sub Set
{
my($self,$key,$val) = @_;

	{
		lock($$self{'_lock_hash'});
		if ( defined($self->{_rh_hash}->{$key}) )
		{
			$self->{_rh_hash}->{$key} = $val;
			return 1;
		}
	}
	my $o_bdb = $$self{'_o_bdb'};
	lock($$self{'_lock_bdb'});
	$o_bdb->db_put($key,$val);

	return 1;
}

# si la cle du hash existe je l'utilise sinon
# je recupere la valeur dans la bdb et je cree la cle dans le hash

sub Concat
{
my($self,$key,$val) = @_;

	{
		if ( defined($self->{_rh_hash}->{$key}) )
		{
			lock($$self{'_lock_hash'});
			$self->{_rh_hash}->{$key} .= $val;
			return 1;
		}
	}
	my $prev=undef;
	{
		my $o_bdb = $$self{'_o_bdb'};
		lock($$self{'_lock_bdb'});
		$o_bdb->db_get($key,$prev);
	}
	if ( ! defined($prev) )
	{
		$self->Set($key,$val);
	}
	else
	{
		$self->Del($key);
		lock($$self{'_lock_hash'});
		$self->{_rh_hash}->{$key} = "$prev$val";
	}
	return 1;
}

# si la cle du hash existe sinon bdb pour limiter les acces a bdb

sub Del
{
my($self,$key) = @_;

	{
		if ( defined($self->{_rh_hash}->{$key}) )
		{
			lock($$self{'_lock_hash'});
			delete($self->{_rh_hash}->{$key});
			return 1;
		}
	}
	lock($$self{'_lock_bdb'});
	my $o_bdb = $$self{'_o_bdb'};
	$o_bdb->db_del($key);
	return 1;
}


sub GetKeys
{
my($self,$which) = @_;

	my @a_keys=();
	{
		if  ( ! defined($which) || $which =~ /hash/i )
		{
			my $rh_hash = $$self{'_rh_hash'};
			lock($$self{'_lock_hash'});
			@a_keys=keys(%{$rh_hash});
		}
	}
	#print STDERR scalar(@a_keys)." keys in hash\n";

	{
		if (! defined($which) || $which =~ /bdb/i )
		{
			lock($$self{'_lock_bdb'});
			my $o_bdb  = $$self{'_o_bdb'};
			my ($k, $v) = ("", "") ;
			my $cursor = $o_bdb->db_cursor() ;
			while ($cursor->c_get($k, $v, DB_NEXT) == 0)
			{ 
				push(@a_keys,$k);
			}
			undef $cursor ;
		}
	}
	#print STDERR scalar(@a_keys)." keys in total\n";
	return @a_keys;
}



1;
