#
# $Id: ScaffGraph.pm 519 2016-08-26 05:40:19Z gouzy $
#

package ScaffGraph;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;
use Contig;
use Digest::MD5 qw(md5 md5_hex md5_base64);
use Scaffold;

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 519 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected=('maxvalidationlength');
	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init();

	foreach my $param (keys(%h_param))
	{
		$self -> _Set ("_$param", $h_param{$param});
	}

	$self->_Set('_rh_graph', undef);
	$self->_Set('_rh_graph_md5', undef);
	$self->_Set('_rh_longestpaths', undef);
	$self->_Set('_graphwithweak', undef);
}



sub InitGraph
{
my($self,$o_linkds,$ra_links,$withweaks) = @_;

	my $digest = md5_hex("$o_linkds,$ra_links,$withweaks");
	if ( defined($self->_FastOneGet('_rh_graph_md5')) && $self->_FastOneGet('_rh_graph_md5') eq $digest )
	{
		$self->_Print("InitGraph: $self not required, digest=$digest\n");
		return $self->_FastOneGet('_rh_graph');
	}

	$self->_Set('_rh_graph_md5',$digest);
	$self->_Print("InitGraph: $self started digest=$digest, ".$self->_FastOneGet('_rh_graph_md5')."\n");

	my $time_av=time;

	$withweaks = 0 if ( ! defined($withweaks) );
	$self->_Set('_graphwithweak',$withweaks);
	my $nb_links_used = 0;
	my @a_links=();
	if ( !defined($ra_links) )
	{
		@a_links = $o_linkds->GetLinks();
		$ra_links=\@a_links;
	}
	else
	{
		print STDERR "InitGraph with a selection\n";
	}
	my %h_key;
	my %h_val;
	my $nb_links = scalar(@$ra_links);
	my %h_graph = ();
	keys(%h_graph) = $nb_links*2;

	for(my $i=0;$i<$nb_links;$i++)
	{
		%h_key = split(/[=\s]+/,$$ra_links[$i]);
		next if (! $o_linkds->GetKV($$ra_links[$i],\%h_val));
		next if( $h_val{weak} == 1 && !$withweaks);
		$nb_links_used++;
		#20121216 desormais je garde les gap negatifs pour pouvoir gerer les chevauchements
		#$h_graph{$h_key{src}}{$h_val{srcext}}{$h_key{tgt}}{$h_val{tgtext}}=&Max(1,$h_val{gaplen_min}).",".&Max(1,$h_val{gaplen_max});
		#$h_graph{$h_key{tgt}}{$h_val{tgtext}}{$h_key{src}}{$h_val{srcext}}=&Max(1,$h_val{gaplen_min}).",".&Max(1,$h_val{gaplen_max});
		($h_val{gaplen_min},$h_val{gaplen_max})=&MinMax($h_val{gaplen_min},$h_val{gaplen_max});
		#print STDERR "$h_key{src}/$h_key{tgt}\n";
		$h_graph{$h_key{src}}{$h_val{srcext}}{$h_key{tgt}}{$h_val{tgtext}}="$h_val{gaplen_min},$h_val{gaplen_max}";
		$h_graph{$h_key{tgt}}{$h_val{tgtext}}{$h_key{src}}{$h_val{srcext}}="$h_val{gaplen_min},$h_val{gaplen_max}";
		$h_graph{$h_key{tgt}}{len}=$h_val{tgtlen};
		$h_graph{$h_key{src}}{len}=$h_val{srclen};
		my $c = "count$h_val{srcext}";
		$h_graph{$h_key{tgt}}{$c}+=$h_val{count};
		$c = "count$h_val{tgtext}";
		$h_graph{$h_key{src}}{$c}+=$h_val{count};
	}
	$self->_Print("InitGraph: #links=$nb_links ; #used=$nb_links_used; elapsed time=".(time-$time_av)."s\n");
	$self->_Set('_rh_graph',\%h_graph);
	return \%h_graph;
}


sub GetLongerNeighbours
{
my($self,$ctg,$ext,$ra_neighbours,$rh_notin) = @_;

	my $rh_graph = $self->_FastOneGet('_rh_graph');
	my $ctglen = $$rh_graph{$ctg}{len};
	foreach my $neighbour (keys(%{$$rh_graph{$ctg}{$ext}}))
	{
		if (! exists($$rh_notin{$neighbour}) && $$rh_graph{$neighbour}{len} > $ctglen )
		{
			push(@$ra_neighbours,$neighbour);
		}
	}
}



sub GetGraph
{
my($self)=@_;

	return $self->_FastOneGet('_rh_graph');
}


sub GetNextContigs
{
my($self,$src,$srcext)=@_;

	my $rh_graph = $self->_FastOneGet('_rh_graph');
	return () if (! $self->Exists($src,$srcext) );
	return sort keys(%{$$rh_graph{$src}{$srcext}});
}

sub GetNextOrientedContigs
{
my($self,$src,$srcext)=@_;

	my $rh_graph = $self->_FastOneGet('_rh_graph');
	return () if (! $self->Exists($src,$srcext) );
	my %h_nextctg = ();
	foreach my $ctg (sort keys(%{$$rh_graph{$src}{$srcext}}))
	{
		my @a_ = keys(%{$$rh_graph{$src}{$srcext}{$ctg}});
		$h_nextctg{$ctg} = "$$rh_graph{$src}{$srcext}{$ctg}{$a_[0]},$a_[0]"; # gaplen_min,gaplen_max,ext (je ne gere qu'une orientation pour le moment)
	}
	return \%h_nextctg;
}

sub Exists
{
my($self,@a_keys) = @_;

	my $rh_graph = $self->_FastOneGet('_rh_graph');
	foreach my $k (@a_keys)
	{
		return &FALSE if ( ! exists($$rh_graph{$k}) );
		$rh_graph = $$rh_graph{$k};
	}
	return &TRUE;
}

sub GetKeys
{
my($self,@a_keys) = @_;

	my $rh_graph = $self->_FastOneGet('_rh_graph');
	foreach my $k (@a_keys)
	{
		return &FALSE if ( ! exists($$rh_graph{$k}) );
		$rh_graph = $$rh_graph{$k};
	}
	return keys(%$rh_graph);
}


# cherche un chemin plus petit que le lien initial

sub FindPath
{
my ($self,$curr,$ext,$target,$targetext,$gaplen,$rh_deja,$ra_path,$o_linkds) = @_;

	my $rh_graph = $self->GetGraph();

	return &FALSE if ( exists($$rh_deja{$curr}) );
	$$rh_deja{$curr} = 1;

	my ($gaplen_min,$gaplen_max) = split(',',$gaplen);

	return &FALSE if ( $gaplen_max < 0 );
	
	my $newext = ( $ext == 5 ) ? 3 : 5;

	#print STDERR "$curr,$ext je cherche $newext gaplen=$gaplen gaplen_min=$gaplen_min gaplen_max=$gaplen_max target=$target >> $$rh_graph{$curr}{$ext}{$target}{$targetext}\n";

	# 20130427 cas particulier ou l'on a un successeur direct
	#if ( $self->Exists($curr,$newext,$target,$targetext) && $gaplen_max > 0 )
	#{
	#	$$rh_deja{$target}=1;
	#	my $ori = ( $targetext eq '5' ) ? 'p' : 'm';	# sequences a coller a droite
	#	unshift(@$ra_path,"$$rh_graph{$curr}{$newext}{$target}{$targetext} #$target$ori$$rh_graph{$target}{len}");
	#	unshift(@$ra_path,"$gaplen #$target$ori$$rh_graph{$target}{len}");
	#	return &TRUE;
	#}


	# tri rajoute le 20130427
	my @a_successors_raw = keys(%{$$rh_graph{$curr}{$newext}});
	my %h_vala =();
	my %h_valb =();
	my @a_successors = sort {
		$o_linkds->GetKV2($curr,$a,\%h_vala);
		$o_linkds->GetKV2($curr,$b,\%h_valb);
		$h_valb{count} = 0 if ( ! defined($h_valb{count}));
		$h_vala{count} = 0 if ( ! defined($h_vala{count}));
		return $h_valb{count} <=> $h_vala{count};
	} @a_successors_raw;
	
	foreach my $next (@a_successors)
	{
		foreach my $nextext ('5', '3')
		{
			next if( ! $self->Exists($curr,$newext,$next,$nextext) );
 			#$o_linkds->GetKV2Oriented($curr,$newext,$next,$nextext,\%h_vala);
 			#print STDERR "\tAP {$curr}{$newext}{$next}{$nextext} $$rh_graph{$curr}{$newext}{$next}{$nextext} $h_vala{count}\n";
			
			my($currnextgaplen_min,$currnextgaplen_max) = split(',',$$rh_graph{$curr}{$newext}{$next}{$nextext});
# 20121216	$currnextgaplen_min = &Max(1,$currnextgaplen_min);

			#my $minlen = $gaplen_min-$currnextgaplen_min-$$rh_graph{$curr}{len};
			#my $maxlen = $gaplen_max-$currnextgaplen_max-$$rh_graph{$curr}{len};

			# 20130404
			my $inserted_len = &Max($currnextgaplen_min,0)+$$rh_graph{$curr}{len};
			my $maxlen = $gaplen_max-$inserted_len;
			my $minlen = &Max($gaplen_min-$inserted_len,0);

			#print STDERR "minlen=$minlen=  Max($gaplen_min-$inserted_len,0)\n";
			#print STDERR "maxlen=$maxlen = $gaplen_max-$inserted_len\n";
	
			# 20130401 je ne peux pas faire ca car je manque des choses que je dois trouver
			# mais j'ai deja fait des retours arriere dans ce coin
			#next if ( $maxlen < 0 );
	
			my $found = &FALSE;
			if ( $maxlen >= 0 )
			{
				if ( $next eq $target && $nextext eq $targetext )
				{
					$$rh_deja{$next}=1;
					$found = &TRUE;
				}
				else
				{
					$found = $self->FindPath($next,$nextext,$target,$targetext,"$minlen,$maxlen",$rh_deja,$ra_path,$o_linkds);
				}
			}
			else
			{
				next;
			}
			if ( $found )
			{
				my $ori = ( $nextext eq '5' ) ? 'p' : 'm';	# sequences a coller a droite
				unshift(@$ra_path,"$$rh_graph{$curr}{$newext}{$next}{$nextext} #$next$ori$$rh_graph{$next}{len}");
				return &TRUE;
			}
		}
	}
	return &FALSE;
}

# l'analyse est toujours faite sur le brin forward uniquement (on gere le brin en revcompl en amont)

sub GetContigsCompatiblesWithGap
{
my ($self,$o_linkds,$src,$srcext,$next,$nextext,$tolerance,$gap) = @_;

	my @a_next = ();
	my $rh_graph = $self->GetGraph();

	my @a_successors_raw = $self->GetKeys($src,$srcext);

	my %h_vala =();
	my %h_valb =();

if ( 0 )
{
	# je donne priorite aux contigs les plus proches
	my @a_successors = sort {
		$o_linkds->GetKV2($src,$a,\%h_vala);
		$o_linkds->GetKV2($src,$b,\%h_valb);
		if ( ! defined($h_valb{count}) || ! defined($h_vala{count}) )
		{
			print STDERR "ValidateOneGraphPath WARNING count undefined count($src,$a)=$h_vala{count} count($src,$b)=$h_valb{count}\n";
			print STDERR Dumper(@a_successors_raw);
		}
		if ( $h_vala{gaplen_min} == $h_valb{gaplen_min} )
		{
			return ($h_vala{gaplen_max}-$h_vala{count}) <=> ($h_valb{gaplen_max}-$h_valb{count});
		}
		return ($h_vala{gaplen_min}-$h_vala{count}) <=> ($h_valb{gaplen_min}-$h_valb{count});
	} @a_successors_raw;
}

	# priorite aux contigs les plus longs
	my @a_successors = sort {
		$o_linkds->GetKV2($src,$a,\%h_vala);
		$o_linkds->GetKV2($src,$b,\%h_valb);
		if ( ! defined($h_valb{count}) || ! defined($h_vala{count}) )
		{
			print STDERR "ValidateOneGraphPath WARNING count undefined count($src,$a)=$h_vala{count} count($src,$b)=$h_valb{count}\n";
			print STDERR Dumper(@a_successors_raw);
		}
		# je ne sais pas qui est src et tgt mais en prenant les deux seul celui qui varie decide du tri
		return ($h_valb{srclen}+$h_valb{tgtlen}) <=> ($h_vala{srclen}+$h_vala{tgtlen});
	} @a_successors_raw;

	# on cherche les succ qui peuvent rentrer dans le gap entre src et next

	# =====SRC=====             ($minsrcnext,$maxsrcnext)                        =====NEXT=====

	# =====SRC=====   ($minsrcsucc,$maxsrcsucc)    ====SUCC====    N/A           =====NEXT=====

	my ($minsrcnext,$maxsrcnext) = split(',',$gap);
	my $gaplen_tol = $maxsrcnext+int($maxsrcnext*$tolerance/100);
	my @a_succ = ();
	foreach my $succ (@a_successors)
	{
		next if ( $succ eq $next );
		next if ( $succ eq $src ); # au cas ou j'accepte les repeats en tandem
		foreach my $extsucc ('5', '3')
		{
			next if( ! $self->Exists($src,$srcext,$succ,$extsucc) );
			my $succlen = $$rh_graph{$succ}{len};
			my $limit_max = &Max(1,$maxsrcnext-$succlen); # c'est la taille max du gap acceptable entre src et succ
			my ($minsrcsucc,$maxsrcsucc) = split(',',$$rh_graph{$src}{$srcext}{$succ}{$extsucc});

			#print STDERR "{$src}{$srcext}{$succ}{$extsucc} $minsrcsucc+$succlen > $gaplen_tol\n";
			next if ( $minsrcsucc+$succlen > $gaplen_tol ); # ca ne rentre pas

			#next if ( $mininit > $mingaplen ); 

			# calcul du gap: succ -> next
			my %h_val =();
			my $newgap = 0;
			my ($minsuccnext,$maxsuccnext) = ();
			if ( $o_linkds->GetKV2Oriented($succ,$extsucc,$next,$nextext,\%h_val) ) # AddRepeat aurait du le trouver
			{
				$minsuccnext = &Min($h_val{gaplen_min},$limit_max);
				$maxsuccnext = &Min($h_val{gaplen_max},$limit_max);
			}
			else
			{
				$minsuccnext=&Max(1,$minsrcnext-$succlen-$minsrcsucc);
				$maxsuccnext=&Max(1,$maxsrcnext-$succlen-$minsrcnext); # j'essaye de ne pas coller
			}
			my($minsuccnext,$maxsuccnext)=&MinMax($minsuccnext,$maxsuccnext);

			# calcul du gap: src -> succ
			$minsrcsucc=&Min($minsrcsucc,$limit_max); # il ne peut rien y avoir de plus long que le gap englobant
			$maxsrcsucc=&Min($maxsrcsucc,$limit_max); # il ne peut rien y avoir de plus long que le gap englobant
			$minsrcsucc=&Max(1,$minsrcsucc);
			($minsrcsucc,$maxsrcsucc)=&MinMax($minsrcsucc,$maxsrcsucc);
		
			my $strand = ( $extsucc eq '5' ) ? 'p' : 'm';
			#print STDERR "$src-$next GAP: ($minsrcnext,$maxsrcnext) $src-$succ $$rh_graph{$src}{$srcext}{$succ}{$extsucc} succlen=$succlen\n";
			#print STDERR "AVANT remise a l'echelle: $src $succ $strand $succlen $minsrcsucc,$maxsrcsucc $minsuccnext,$maxsuccnext\n";
			# ensuite je remets a l'echelle pour que la somme $maxsrcsucc+$succlen+$maxsuccnext ne soit pas suprieure a $maxsrcnext
			my $fact=($maxsrcsucc+$maxsuccnext)/&Max(1,$maxsrcnext-$succlen);
			$minsrcsucc  = &Max(1,int($minsrcsucc/$fact));
			$maxsrcsucc  = &Max(1,int($maxsrcsucc/$fact));
			$minsuccnext = &Max(1,int($minsuccnext/$fact));
			$maxsuccnext = &Max(1,int($maxsuccnext/$fact));
			#print STDERR "APRES remise a l'echelle: $src $succ $strand $succlen $minsrcsucc,$maxsrcsucc $minsuccnext,$maxsuccnext\n";

			push(@a_succ,"$succ $strand $succlen $minsrcsucc,$maxsrcsucc $minsuccnext,$maxsuccnext");
		}
	}
	return \@a_succ;
}


sub ValidateOneGraphPath
{
my($self,$o_linkds,$o_scaffolder,$src,$srcext,$next,$nextext,$gap,$tolerance,$confirm)=@_;


	my $rh_graph = $self->GetGraph();
	$tolerance = 0 if (! defined($tolerance) );	# lorsque je cherche un chemin court j'ai plus de mal a inserer les repeats
	my $o_newpath = undef;
	my $str_newpath = undef;
	if ( $confirm  && ! $self->Exists($src,$srcext,$next,$nextext) )
	{
		return $o_newpath; # ca peut arriver quand j'essaye de combler les trous a la fin
	}
	my @a_successors_raw = $self->GetKeys($src,$srcext);

	# le tri les successeurs selon une taille de gap croissant 20120215 n'amliore pas
	# je tri sur le count decroissant
	my @a_successors=();

	my %h_vala =();
	my %h_valb =();
	@a_successors = sort {
		$o_linkds->GetKV2($src,$a,\%h_vala);
		$o_linkds->GetKV2($src,$b,\%h_valb);
		$h_valb{count}=0 if ( ! defined($h_valb{count}) );
		$h_vala{count}=0 if ( ! defined($h_vala{count}) );
		return $h_valb{count} <=> $h_vala{count};
	} @a_successors_raw;


	foreach my $other (@a_successors)
	{
		next if( $other eq $next ); # je veux un autre successeur que celui que je cherche
		foreach my $extother ('5', '3')
		{
			next if( ! $self->Exists($src,$srcext,$other,$extother) );
			# gap size = gaplen with next - gaplen - other

			my ($mininit,$maxinit) = ( ! defined($gap) ) ? split(',',$$rh_graph{$src}{$srcext}{$next}{$nextext}) : split(',',$gap);
			$mininit=&Max(1,$mininit);
			if ( $tolerance )
			{
				$mininit += int($mininit*$tolerance/100);
				$maxinit += int($maxinit*$tolerance/100);
			}
			my ($minsrcother,$maxsrcother) = split(',',$$rh_graph{$src}{$srcext}{$other}{$extother});
			$minsrcother=&Max(1,$mininit);
			my $mindistance = $mininit-$minsrcother;
			#  20130427 my $maxdistance = $maxinit-$maxsrcother;
			my $maxdistance = $maxinit-$minsrcother-$$rh_graph{$other}{len};
			#print STDERR ">>$src $other,$extother,$next,$nextext,$tolerance distance=$mindistance,$maxdistance\n";
			my %h_deja=();
			$h_deja{$src}=1;
			$h_deja{$next}=1;
			my @a_path=();
			if ( $maxdistance > 0 )
			{
				# je vais regarder dans la liste des successeurs de src si je trouve un autre chemin qui mene a next
				# le changement d'extremite pour other est fait dans la fonction recursive
				if ( $self->FindPath($other,$extother,$next,$nextext,"$mindistance,$maxdistance",\%h_deja,\@a_path,$o_linkds) )
				{
					my $srcori    = ( $srcext eq '3' ) ? 'p' : 'm';	 # liens de gauche a droite
					$str_newpath  = "#$src$srcori$$rh_graph{$src}{len} $$rh_graph{$src}{$srcext}{$other}{$extother}";
					my $otherori  = ( $extother eq '5' ) ? 'p' : 'm'; # pour les sequences a coller a droite c'est different
					$str_newpath .= " #$other$otherori$$rh_graph{$other}{len}";
					$str_newpath .= " ".join(' ',@a_path);
					#print STDERR "FindPath: $str_newpath\n";
				}
				# je fais ca car je n'arrive pas bien a gere le cas dans la fonction recursive
				else
				{
					my $newextother = ( $extother == 5 ) ? 3 : 5;
					if ( $self->Exists($other,$newextother,$next,$nextext) )
					{
						$h_deja{$other}=1;
						my $srcori    = ( $srcext eq '3' ) ? 'p' : 'm';	 # liens de gauche a droite
						$str_newpath  = "#$src$srcori$$rh_graph{$src}{len} $$rh_graph{$src}{$srcext}{$other}{$extother}";
						my $otherori  = ( $extother eq '5' ) ? 'p' : 'm'; # pour les sequences a coller a droite c'est different
						$str_newpath .= " #$other$otherori$$rh_graph{$other}{len} $$rh_graph{$other}{$newextother}{$next}{$nextext}";
						my $nextori   = ( $nextext eq '5' ) ? 'p' : 'm';	# sequences a coller a droite
						$str_newpath .= " #$next$nextori$$rh_graph{$next}{len}";
						#print STDERR "Next: $str_newpath\n";
					}
				}

				if ( defined($str_newpath) )
				{
					my $id = "TMP".time;
					$o_newpath    = New Scaffold('name'=>$id,'maxvalidationlength'=>$self->_FastOneGet('_maxvalidationlength'));
					if ( !$o_newpath->InitPathFromString(\$str_newpath,$o_scaffolder,&FALSE) )
					{
						$o_newpath->DeregisterContigs();
						return undef;
					}
					#$o_newpath->SetContigAnchors($rh_anchor_contigs);
					$o_newpath->InitAnchor(&FALSE);
					#my ($anchor) = ( defined($o_newpath->GetAnchor()) ) ? $o_newpath->GetAnchor() : "";
					#print STDERR "ValidateOneGraphPath ANCHOR=$anchor\n";
					if ( ! $o_newpath->IsAnchoringConsistent() )
					{
						$self->_Print("ValidateOneGraphPath anchoring inconsistency found and skipped in $str_newpath\n");
						$o_newpath->DeregisterContigs();
						return undef;
					}
					#print STDERR "AV $src->$next:".$o_newpath->Get()."\n";
					my ($mininit,$maxinit) = ( ! defined($gap) ) ? split(',',$$rh_graph{$src}{$srcext}{$next}{$nextext}) : split(',',$gap);
					$o_newpath->ResetGapLen(undef,undef,"$mininit,$maxinit");
					#print STDERR "AP $src->$next:".$o_newpath->Get()."\n";
					return $o_newpath;
				}
			}
		}
	}
	return $o_newpath;
}

# le plus long en terme de noeuds
# attention je peux passer par des liens faibles selon ce qui est fait dans ValidateOneGraphPath 

sub FindLongestPath
{
my($self,$o_scaff,$o_linkds,$o_scaffolder,$gaplentolerance,$maxpass)=@_;

	my %h_deja=();

	#my $o_prevpath = $o_scaff->Clone();
	my $o_prevpath = $o_scaff;

	my $fini = &FALSE;

	my $graphtype = $self->_FastOneGet('_graphwithweak');
	my $rh_longestpaths = $self->_FastOneGet('_rh_longestpaths');

	my $nb_pass=(defined($maxpass))?$maxpass:&MAXINT;
	my $signature_suffix = "$gaplentolerance###$nb_pass###$graphtype";
	my $digest = md5_hex($o_scaff->Get()."###$signature_suffix");
	if ( exists($$rh_longestpaths{$digest}) )
	{
		#deja un LongestPath dans ce contexte
		#$self->_Print("Already a longestpath $digest $path###$signature_suffix\n");
		return undef;
	}

	my $o_newpath = $o_scaff->Clone();
	$o_newpath->SetContigs();

	#print STDERR $o_scaff->Get()."\n";

	while( ! $fini && $nb_pass )
	{
		$nb_pass--;
		my $ra_path = $o_prevpath->GetArray();
		my @a_newpath = ();
		push(@a_newpath,$$ra_path[0]);
		my $nb = scalar(@$ra_path);
		my $av = time;
		my %h_val=();
		for(my $i=0;$i<$nb-2;$i+=2)  # srccontig gap nextcontig
		{
			my $gaplen = $$ra_path[$i+1];
			my ($src,$srcext,$next,$nextext) = $$ra_path[$i]->ConvertPathLink2GraphLink($$ra_path[$i+2],&FALSE);
			my $key = "$src,$srcext,$next,$nextext";
			my $o_longestlocalpath = undef;
			my $overlap = &FALSE;
			if ( $o_linkds->GetKV2Oriented($src,$srcext,$next,$nextext,\%h_val) )
			{
				$overlap=$h_val{ovllen} || &FALSE;
			}
			#print STDERR "Searching: $src,$srcext,$next,$nextext,$gaplen,$gaplentolerance\n";
			$o_longestlocalpath = $self->ValidateOneGraphPath($o_linkds,$o_scaffolder,$src,$srcext,$next,$nextext,$gaplen,$gaplentolerance,&FALSE) if ( ! exists($h_deja{$key}) && ! $overlap );
			my $new_path=&FALSE;
			if( defined($o_longestlocalpath) )
			{
				#print STDERR "LONGEST AV ".$o_longestlocalpath->Get()."\n";
				$o_longestlocalpath->ResetGapsOnePath($o_linkds);
				#print STDERR "LONGEST AP ".$o_longestlocalpath->Get()."\n";
				#print STDERR "Length ".$o_longestlocalpath->GetLength()."\n";
				#print STDERR "Internal Length ".$o_longestlocalpath->GetInternalLength()." full=".$o_longestlocalpath->GetInternalLength(&TRUE)."\n";
				if ( $o_scaff->AreAnchoringCompatibles($o_longestlocalpath) ) # ca peut arriver 
				{
					$new_path=&TRUE;
					my %h_deja_in_path =();
					my $ra_found=$o_longestlocalpath->GetArray();
					for(my $j=0;$j<scalar(@$ra_found);$j+=2)
					{
						my $nm = $$ra_found[$j]->GetName();
						if ( exists($h_deja_in_path{$nm}) )
						{
							$new_path = &FALSE;
							last;
						}
						$h_deja_in_path{$nm}=1;
					}
					if ($new_path)
					{	
						for(my $j=1;$j<scalar(@$ra_found);$j++)
						{
							push(@a_newpath,$$ra_found[$j]);
						}
					}
				}
				$o_longestlocalpath->DeregisterContigs();
			}
			$h_deja{$key}=1;
			if (! $new_path )
			{
				push(@a_newpath,$$ra_path[$i+1],$$ra_path[$i+2]); # gap nextcontig
			}
		}
		my $exttime = time;
#		print STDERR "nb_pass=$nb_pass EXTEND=".($exttime-$av)."\n";
		$o_newpath->Reset();
		$o_newpath->Push(@a_newpath);
		$o_newpath->SetContigs();
		$o_newpath->InitAnchor(&FALSE);
		if ( $o_newpath->IsContainingTandemRepeat() || ! $o_newpath->IsAnchoringConsistent(&FALSE) )
		{
			$self->_Print("Repeat trouve dans ".$o_newpath->Get().", je n'utilise pas\n");
			$o_newpath->DeregisterContigs();
			# newpath est celui que je retourne  mais celui que je veux c'est le precedent (pour lequel le SetContigs est deja fait en principe)
			$o_newpath = $o_prevpath;
			$o_newpath->SetContigs();
			$fini = &TRUE;
		}
		if(  $o_newpath->Get() eq $o_prevpath->Get() )
		{
			# newpath est celui que je retourne  mais celui que je veux c'est le precedent (pour lequel le SetContigs est deja fait en principe)
			$o_newpath->DeregisterContigs();
			$o_newpath = $o_prevpath;
			$o_newpath->SetContigs();
			$fini = &TRUE;
		}
		else # le nouveau est bon je switche
		{
			$o_prevpath->DeregisterContigs();
			$o_prevpath = $o_newpath;
			$o_newpath  = $o_prevpath->Clone;
			$o_newpath->SetContigs();
			#print STDERR $o_newpath->Get()."\n";
		}
	}
	$digest = md5_hex($o_newpath->Get()."###$signature_suffix");
	$$rh_longestpaths{$digest}=1;
	if ( $o_newpath->Get() ne $o_scaff->Get() )
	{
		$self->_Print("FindLongestPath: ORI:>".$o_scaff->Get()."<\nFindLongestPath: NEW:>".$o_newpath->Get()."<\n");
		return $o_newpath;
	}
	return undef;
}

sub FindLongestPathString
{
my($self,$o_scaff,$o_linkds,$o_scaffolder,$gaplentolerance,$maxpass)=@_;

	my $o_longestnewpath = $self->FindLongestPath($o_scaff,$o_linkds,$o_scaffolder,$gaplentolerance,$maxpass);
	if ( defined($o_longestnewpath) )
	{
		return $o_longestnewpath->Get();
	}
	return undef;
}



sub AssertParams
{
my ($self,$rh_param,$ra_expected) = @_;
	
	foreach my $param (@{$ra_expected})
	{
		if ( !defined($$rh_param{$param}) )
		{
			print STDERR Dumper(%$rh_param);
			Carp::croak("Fatal error in constructor: $param must be provided\n") 
		}
	}
}

sub _Print
{
my($self,$msg)=@_;

	print STDERR "$msg";
}

1;
