#
# $Id: SequenceSet.pm 220 2013-12-20 14:48:08Z gouzy $
#

package SequenceSet;

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;

use File::Temp;

use threads;
use threads::shared;



# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 220 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ('infile','type');
	if ($h_param{paired} ne 'no')
	{
		push(@a_expected,'pattern');
	}
	$self->AssertParams(\%h_param,\@a_expected);

	$self->SUPER::_Init();

	foreach my $param (keys(%h_param))
	{
		$self -> _Set ("_$param", $h_param{$param});
	}
}

=head2 procedure Load

 Title        : Load
 Usage        : $o_hspds -> Load;

=cut

sub Load 
{
	my ($self,%h_param) = @_;

	my $cpt=0;
	my $what = (defined($h_param{what}) ) ? $h_param{what} : 'all';
	my $filein = $self->_FastOneGet('_infile');
	if ( $what eq 'noseq' && defined($h_param{verbose}) && $h_param{verbose} )
	{
		print STDERR "Sequences not loaded for dataset >$filein<\n";
		return 0;
	}
	my $f_inf = &GetStreamIn($filein);
	my ($cpt_in,$cpt_out)=(0,0);
	my $seq=undef;
	my $id=undef;
	my $type = $self->_FastOneGet('_type');
	my $mode = 2; # len+seq ( $what eq 'all')
	$mode = 1  if ( $what eq 'seq' ); # seq only
	$mode = -2 if ( $what eq 'len' ); # len only
	while (my $l = <$f_inf>)
	{	
		chomp($l);
		if ( $l =~ /^>(\S+)/ )
		{
			if ( defined($seq) ) 
			{
				&CleanSeq(\$seq,$type);
				$self->SetLen($id,length($seq)) if ( abs($mode) == 2 );
				$self->SetSeq($id,$seq)         if ( $mode > 0 );
				$cpt++;
			}
			$id = $1;
			$seq = '';
		}
		else
		{
			$seq .= $l;
		}
	}
	if ( defined($seq) ) 
	{
		&CleanSeq(\$seq,$type);
		$self->SetLen($id,length($seq)) if ( abs($mode) == 2 );
		$self->SetSeq($id,$seq)         if ( $mode > 0 );
		$cpt++;
	}
	$f_inf->close;
	$self->_Set('count',$cpt);
	if ( defined($h_param{verbose}) && $h_param{verbose} )
	{
		print STDERR "Seq in: $cpt\n";
	}
	return $cpt;
}

sub AssertParams
{
my ($self,$rh_param,$ra_expected) = @_;
	
	foreach my $param (@{$ra_expected})
	{
		if ( !defined($$rh_param{$param}) )
		{
			print STDERR Dumper(%$rh_param);
			Carp::croak("Fatal error in constructor: $param must be provided\n") 
		}
	}
}

sub Add
{
my($self,$key,$r_seq) = @_;

	$self->SetSeq($key,$$r_seq);
	$self->SetLen($key,length($$r_seq));
}

sub DumpFastaShuffled
{
my($self,$f_pe,$f_se) = @_;

	my $pattern = $self->_FastOneGet('_pattern');
	my @a_ids = $self->GetSeqIds();
	my %h_insert=();
	foreach my $seqid (@a_ids)
	{
		my ($insert,$ext) = $seqid =~ /$pattern/;
		$h_insert{$insert}{$ext}=$seqid;
	}
	foreach my $insert (sort keys(%h_insert))
	{
		my @a_keys = sort keys(%{$h_insert{$insert}});
		if ( $#a_keys == 0 )
		{
			print $f_se ">$h_insert{$insert}{$a_keys[0]}\n".$self->GetSeq($h_insert{$insert}{$a_keys[0]})."\n";
		}
		else
		{
			foreach my $key (@a_keys)
			{
				print $f_pe ">$h_insert{$insert}{$key}\n".$self->GetSeq($h_insert{$insert}{$key})."\n";
			}
		}
	}
}



sub DumpFasta
{
my($self,$filename) = @_;

	my $f_out = &GetStreamOut($filename);
	my @a_ids = $self->GetSeqIds();
	foreach my $seqid (sort @a_ids)
	{
		print $f_out ">$seqid\n".&FormatSeq(\$self->GetSeq($seqid))."\n";
	}
	$f_out->close;

}
1;
