#
# $Id: BerkeleyDB.pm 246 2014-01-03 09:43:50Z gouzy $
#

package SequenceSet::BerkeleyDB;

use SequenceSet;
use base qw( SequenceSet );

use General;
use GeneralBioinfo;
use BerkeleyDB;
use File::Basename;
use File::Temp qw/ tempfile tempdir /;

use threads;
use threads::shared;

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 246 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	$self->SUPER::_Init(%h_param);

	if ( ! defined($ENV{TMPDIR}) )
	{
		Carp::croak("Error: TMPDIR is required for using BerkeleyDB storage\n");
	}
	my $ftmp = tempdir ( TEMPLATE => 'lynx-contigs-XXXX', DIR => $ENV{TMPDIR}, CLEANUP=> 0);
	my $dir = $ftmp;

	my $env = new BerkeleyDB::Env 
		-Cachesize => 1000000000,
		-TxMax     => 1000,
		-MaxLocks  => 10000,
		-MaxObjects => 10000,
		#  -ErrFile   => "berkeley.err",
		#  -Verbose   => 1,
		-Home   => $dir,
		# rajouter DB_THREAD est tres problematique
		-Flags  => DB_CREATE| DB_INIT_CDB | DB_INIT_MPOOL or die "cannot open environment: $BerkeleyDB::Error\n";

	my %h_db=();

	my ($fseq) = File::Temp->new( TEMPLATE => 'lynx-seq-XXXX', DIR => $dir, SUFFIX => '.db');
	$fseq = $fseq->filename;

	my $db_seq = new BerkeleyDB::Hash(
		-Flags  => DB_CREATE,
		-Env       => $env,
		-Filename  => $fseq) or die "ERROR, unable to create >$fseq\n";

	my ($flen) = File::Temp->new( TEMPLATE => 'lynx-len-XXXX', DIR => $dir, SUFFIX => '.db');
	$flen = $flen->filename;

	my $db_len = new BerkeleyDB::Hash(
		-Flags  => DB_CREATE,
		-Env       => $env,
		-Filename  => $flen) or die "ERROR, unable to create >$flen\n";

	$self ->_Set('_db_seq', $db_seq);
	$self ->_Set('_db_len', $db_len);
}

sub Reset
{
my($self)=@_;

	my $db_seq = $$self{'_db_seq'};
	my ($k, $v) = ("", "") ;
	my $cursor = $db_seq->db_cursor() ;
	while ($cursor->c_get($k, $v, DB_NEXT) == 0)
	{
		$db_seq->db_del($k);
	}
	undef $cursor ;
	my $db_len = $$self{'_db_len'};
	my $cursor = $db_len->db_cursor() ;
	while ($cursor->c_get($k, $v, DB_NEXT) == 0)
	{
		$db_len->db_del($k);
	}
	undef $cursor ;
}


sub SetLen
{
my($self,$key,$val) = @_;

	my $db_len = $$self{'_db_len'};
	$db_len->db_put($key,$val);
	return 1;
}

sub GetLen
{
my($self,$key) = @_;

	my $val;
	my $db_len = $$self{'_db_len'};
	$db_len->db_get($key,$val);
	return $val;
}

sub SetSeq
{
my($self,$key,$val) = @_;

	my $db_seq = $$self{'_db_seq'};
	$db_seq->db_put($key,$val);
	return 1;
}

sub CatSeq
{
my($self,$key,$val) = @_;

	my $db_seq = $$self{'_db_seq'};
	my $valprev = $self->GetSeq($key);
	$db_seq->db_put($key,"$valprev$val");
	return 1;
}


sub GetSeq
{
my($self,$key) = @_;
	
	my $val;
	my $db_seq = $$self{'_db_seq'};
	$db_seq->db_get($key,$val);
	return $val;
}

sub Del
{
my($self,$key) = @_;

	my $db_seq = $$self{'_db_seq'};
	my $db_len = $$self{'_db_len'};
	$db_seq->db_del($key);
	$db_len->db_del($key);
	return 1;
}

sub GetSeqIds
{
my($self,$key) = @_;
	
	my $db_seq = $$self{'_db_seq'};
	my @a_keys=();
	my ($k, $v) = ("", "") ;
	my $cursor = $db_seq->db_cursor() ;
	my $cpt=0;
	while ($cursor->c_get($k, $v, DB_NEXT) == 0)
	{ 
		$cpt++;
		push(@a_keys,$k);
	}
	undef $cursor ;
	# au cas ou je n'ai que les longueurs
	if ( $cpt == 0 )
	{
		my $cursor = $db_seq->db_cursor() ;
		my $cpt=0;
		while ($cursor->c_get($k, $v, DB_NEXT) == 0)
		{ 
			push(@a_keys,$k);
		}
		undef $cursor ;
	}
	return @a_keys;
}


sub GetRefOnSeq
{
my($self,$key) = @_;

	# je ne sais pas si c'est pertinent avec une berkeleydb car on ne peut pas modifier la base
	my $val = $self->GetSeq($key);
	return \$val;
}

sub PrintInfo
{
my($self) = @_;

	print  STDERR "Filename: ".$self->_FastOneGet('_infile')."\n";
	my $nb = $self->GetSeqIds();
	printf STDERR "Number of sequences loaded: %d\n",$nb;
	return $nb;
}

1;



