#
# $Id: shared.pm 220 2013-12-20 14:48:08Z gouzy $
#

# pour le multithreading il ne faut pas que utiliser le destructeur standard
sub BerkeleyDB::Hash::DESTROY
{
	; #print STDERR "Destroy BerkeleyDB::Hash::DESTROY\n";
}

sub BerkeleyDB::Env::DESTROY
{
	; #print STDERR "Destroy BerkeleyDB::Env::DESTROY\n";
}

package SequenceSet::BerkeleyDB::shared;

use SequenceSet;
use SequenceSet::BerkeleyDB;
use base qw(SequenceSet::BerkeleyDB);

use General;
use GeneralBioinfo;
use BerkeleyDB;

use threads;
use threads::shared;

# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 220 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ('lock_len','lock_seq');
	$self->AssertParams(\%h_param,\@a_expected);

	if ( ! is_shared($h_param{lock_len}) || ! is_shared($h_param{lock_seq}) )
	{
		Carp::croak("Error: lock_len and lock_seq must be shared variables\n");
	}

	$self->SUPER::_Init(%h_param);

}


sub SetLen
{
my($self,$key,$val) = @_;

	my $db_len = $$self{'_db_len'};
	lock($$self{'_lock_len'});
	$db_len->db_put($key,$val);
	return 1;
}

sub GetLen
{
my($self,$key) = @_;

	my $val;
	my $db_len = $$self{'_db_len'};
	lock($$self{'_lock_len'});
	$db_len->db_get($key,$val);
	return $val;
}


sub SetSeq
{
my($self,$key,$val) = @_;

	my $db_seq = $$self{'_db_seq'};
	lock($$self{'_lock_seq'});
	$db_seq->db_put($key,$val);
	return 1;
}

sub CatSeq
{
my($self,$key,$val) = @_;

	my $db_seq = $$self{'_db_seq'};
	my $valprev;
	lock($$self{'_lock_seq'});
	$db_seq->db_get($key,$valprev);
	$valprev = '' if ( ! defined($valprev) );
	$db_seq->db_put($key,"$valprev$val");
	return 1;
}

sub GetSeq
{
my($self,$key) = @_;
	
	my $val;
	my $db_seq = $$self{'_db_seq'};
	lock($$self{'_lock_seq'});
	$db_seq->db_get($key,$val);
	return $val;
}

sub Del
{
my($self,$key) = @_;

	my $db_seq = $$self{'_db_seq'};
	my $db_len = $$self{'_db_len'};
	lock($$self{'_lock_seq'});
	lock($$self{'_lock_len'});
	$db_seq->db_del($key);
	$db_len->db_del($key);
	return 1;
}

1;



