#
# $Id: Hash.pm 220 2013-12-20 14:48:08Z gouzy $
#

package SequenceSet::Hash;

use SequenceSet;
use base qw( SequenceSet );

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;

use threads;
use threads::shared;


# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 220 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	$self->SUPER::_Init(%h_param);

	my %h_seq:shared=();
	$self -> _Set ('_rh_seq', \%h_seq);
	my %h_len:shared=();
	$self -> _Set ('_rh_len', \%h_len);
}


sub Reset
{
my($self)=@_;

	%{$self->{_rh_len}}=();
	%{$self->{_rh_seq}}=();
	return 1;
}

sub SetLen
{
my($self,$key,$val) = @_;

	$self->{_rh_len}->{$key} = $val;
	return 1;
}

sub GetLen
{
my($self,$key) = @_;

	return $self->{_rh_len}->{$key};
}

sub SetSeq
{
my($self,$key,$val) = @_;

	$self->{_rh_seq}->{$key} = $val;
	return 1;
}


sub GetSeq
{
my($self,$key) = @_;

	return $self->{_rh_seq}->{$key};
}

sub CatSeq
{
my($self,$key,$val) = @_;

	$self->{_rh_seq}->{$key} .= $val;
	return 1;
}


sub Del
{
my($self,$key) = @_;

	delete($self->{_rh_seq}->{$key});
	delete($self->{_rh_len}->{$key});
	return 1;
}

sub GetSeqIds
{
my($self,$key) = @_;

	my $r_keys=$self->{'_rh_seq'};
	return keys(%$r_keys);
}


sub GetRefOnSeq
{
my($self,$key) = @_;

	return \$self->{_rh_seq}->{$key};
}


sub PrintInfo
{
my($self) = @_;

	print  STDERR "Filename: ".$self->_FastOneGet('_infile')."\n";
	my $r_keys=$self->{'_rh_seq'};
	my $nb = keys(%$r_keys);
	if ( $nb == 0 )
	{
		$r_keys=$self->{'_rh_len'};
		$nb = keys(%$r_keys);
	}
	printf STDERR "Number of sequences loaded: %d\n",$nb;
	return $nb;
}


1;
