#
# $Id: shared.pm 220 2013-12-20 14:48:08Z gouzy $
#

package SequenceSet::Hash::shared;

use SequenceSet;
use SequenceSet::Hash;
use base qw( SequenceSet::Hash);

use strict;
use General;
use GeneralBioinfo;
use base qw( LipmObject );
use Data::Dumper;

use threads;
use threads::shared;


# Copyright INRA
# Jerome.Gouzy@toulouse.inra.fr

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 220 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead

=cut

sub _Init
{
	my ($self,%h_param) = @_;

	my @a_expected = ('lock_len','lock_seq');
	$self->AssertParams(\%h_param,\@a_expected);

	if ( ! is_shared($h_param{lock_len}) || ! is_shared($h_param{lock_seq}) )
	{
		Carp::croak("Error: lock_len and lock_seq must be shared variables\n");
	}

	$self->SUPER::_Init(%h_param);
}

sub SetLen
{
my($self,$key,$val) = @_;

	lock($$self{'_lock_len'});
	$self->{_rh_len}->{$key} = $val;
	return 1;
}

sub GetLen
{
my($self,$key) = @_;

	lock($$self{'_lock_len'});
	return $self->{_rh_len}->{$key};
}


sub SetSeq
{
my($self,$key,$val) = @_;

	lock($$self{'_lock_seq'});
	$self->{_rh_seq}->{$key} = $val;
	return 1;
}

sub GetSeq
{
my($self,$key) = @_;

	lock($$self{'_lock_seq'});
	return $self->{_rh_seq}->{$key};
}


sub CatSeq
{
my($self,$key,$val) = @_;

	lock($$self{'_lock_seq'});
	$self->{_rh_seq}->{$key} .= $val;
	return 1;
}

sub Del
{
my($self,$key) = @_;

	lock($$self{'_lock_seq'});
	lock($$self{'_lock_len'});
	delete($self->{_rh_seq}->{$key});
	delete($self->{_rh_len}->{$key});
	return 1;
}

1;
