
# prerequesite: file sorted on the target id

sub GetListOfOneEndMapped
{
my ($file,$fileout,$rh_oneend,$querycol)=@_;

	my $prev_id = '';
	my @a_reads = ();
	my $f_in = &GetStreamIn($file);
	my $f_out = &GetStreamOut("$fileout.tmp");
	my %h_res=();
	my $cpt = 0;
	while(my $l=<$f_in>)
	{
		chomp($l);
		$cpt++;
		&ParseM8Line(\$l,$querycol,\%h_res);
		my $insert = &GetInsert($h_res{target});
		if ( $insert ne $prev_id && scalar(@a_reads) )
		{
			if ( scalar(@a_reads) == 1 )
			{
				$$rh_oneend{$prev_id}{HSP} = $a_reads[0]; 
				print $f_out "$a_reads[0]\n";
			}
			@a_reads=();
		}
		push(@a_reads,$l);
		$prev_id = $insert;
	}
	$f_in->close;
	$f_out->close;
	if ( $querycol == 2 )
	{
		&UnixSort("$fileout.tmp",$fileout,['2','9n']);
	}
	else
	{
		&UnixSort("$fileout.tmp",$fileout,['1','7n']);
	}
	unlink "$fileout.tmp";
	print STDERR "$cpt HSPs read\n";
	print STDERR scalar(keys(%$rh_oneend))." HSPs loaded\n";
}


sub DumpIntervalInBedformat
{
my($filem8,$filebed,$rh_oneend,$rh_scaff,$querycol,$default_insertlen,$reverse_seq)=@_;
	
	my $f_in = &GetStreamIn($filem8);
	my $f_bed = &GetStreamOut($filebed);
	my %h_res = ();
	while (my $l=<$f_in>)
	{
		chomp($l);
		&ParseM8Line(\$l,$querycol,\%h_res);
		my $insert = &GetInsert($h_res{target});
		my $ins_len = (defined($$rh_oneend{$insert}{INSERTLEN}) ) ? $$rh_oneend{$insert}{INSERTLEN} : $default_insertlen;	
		#print "$h_res{qstart}\t$h_res{qend}\t$h_res{query}\t$h_res{strand}\n";
		my ($min,$max)=(0,0);
		if (  $h_res{strand} eq '+' || $reverse_seq )
		{
			$min = $h_res{qstart}-1;
			$max = $h_res{qstart}+$ins_len;
		}
		else
		{
			$min = $h_res{qend}-$ins_len;
			$max = $h_res{qend};
		}
		my $strout = "$h_res{query}\t$min\t$max\t$h_res{target}--$h_res{qstart}--$h_res{qend}--$ins_len";
		if ( $min < 0 || $max > $$rh_scaff{$h_res{query}}{len} )
		{
			#print STDERR "min=0, max=$$rh_scaff{$h_res{query}}{len} $strout\n";
		}
		else
		{
			print $f_bed "$strout\n";
		}
	}
	$f_bed->close;
	$f_in->close;
}


sub GetListOfGaps
{
my($fasta,$gff3,$rh_scaff,$rh_gaps)=@_;

	my $f_out = &GetStreamOut($gff3);
	my $cpt=0;
	foreach my $id (sort keys(%$rh_scaff))
	{
		my $seq = $$rh_scaff{$id}{sequence};
		my @a_seq = split('',$seq);
		my $found=&FALSE;
		my $start = 0;
		$cpt=1;
		#Mt3.5.1Chr0     H       exon    10830   14480   .       -       .       ID=AC130806_4.1_exon_2;Parent=AC130806_4.1
		for(my $i=0;$i<scalar(@a_seq);$i++)
		{
			if ( $a_seq[$i] =~ /N/i )
			{
				if ( ! $found )	
				{
					printf $f_out "$id\t.\t.\t%d",$i+1;
					$start = $i+1;
					$found=&TRUE;
				}
			}
			elsif ($found)
			{	
				my $gid = sprintf("GAP%s",$cpt++);
				printf $f_out "\t%d\t.\t.\t.\tID=$gid.$$;Name=$gid\n",$i;
				$$rh_gaps{$id} .= sprintf("$id--$gid-$start-%d\n",$i);
				$found = &FALSE;
			}
		}
		if ( $found )
		{
			my $gid = sprintf("GAP%s",$cpt++);
			print $f_out "\t%d\t.\t.\t.\tID=$gid.$$;Name=$gid",$$rh_scaff{$id}{len};
			$$rh_gaps{$id} .= "-$$rh_scaff{$id}{len}\n";
		}
	}
	$f_out->close;
}


sub DumpReadsInGaps
{
my($outdir,$fileovl,$reads,$mates,$rh_oneend,$default_insertlen,$ext)=@_;

	# seq1    .       .       6475    6758    .       .       .       ID=GAP14760.0;Name=GAP14760.0   seq1    13      13014   PHOSPHORE_0149:7:2107:9373:110946#CTTGTA/1      284
	my $f_in = &GetStreamIn($fileovl);
	my $prev_gaprange = '';
	my $f_lstmp = undef;
	my $f_lstpe = undef;
	$ext = ".$ext" if ( defined($ext) );
	while(my $l=<$f_in>)
	{
		chomp($l);
		@_ = split("\t",$l);
		my ($gid) = $_[8] =~ /Name=([^\;]+)/;
		my $gaprange = "$gid-$_[3]-$_[4]";
		my $gapsize = $_[4]-$_[3]+1;
		my $distmin = abs($_[10]-$_[3]);
		my $distmax = abs($_[11]-$_[4]);
		#print STDERR "$l\n$gaprange, $gapsize, $distmin, $distmax\n";
		if ( $gaprange ne $prev_gaprange )
		{
			$f_lstmp->close if ( defined($f_lstmp) );
			$f_lstpe->close if ( defined($f_lstpe) );
			my $outd = "$outdir/$_[0]--$gaprange";
			mkdir "$outd" if  ( ! -d $outd );
			$f_lstmp = &GetStreamOut("$outd/$_[0]--$gaprange.lst.mp$ext");
			$f_lstpe = &GetStreamOut("$outd/$_[0]--$gaprange.lst.pe$ext");
		}
		# pour eviter le bruit du aux grands insert, l'une des deux extremites doit etre au bord du trou
		my $end = $_[12];
		my ($otherend) = ( $end =~ /\/1/ ) ? "2" : "1";
		$end =~ s/--.+//;
		$end =~ s/\d$/$otherend/;
		my $insert = &GetInsert($end);
		my $insert_len = (defined($$rh_oneend{$insert}{INSERTLEN}) ) ? $$rh_oneend{$insert}{INSERTLEN} : $default_insertlen;	
		my $margin = $insert_len/2;
		#if ( $distmin < $margin || $distmax < $margin )
		if ( 1 )
		{
			if ( $insert_len < 1000 )
			{
				print $f_lstpe "$end\n";
			}
			else
			{
				print $f_lstmp "$end\n";
			}
		}	
		$prev_gaprange = $gaprange;
	}
	$f_lstmp->close if ( defined($f_lstmp) );
	$f_lstpe->close if ( defined($f_lstpe) );
	$f_in->close;
}



sub System
{
my($cde)=@_;

	print STDERR "$cde\n" if ( $DEBUG );	
	system($cde);
}



sub GetInsert
{
my($str)=@_;

	$str=~s/\/\d.*$//;
	return $str;
}

1;
