use strict;
use warnings;

#
# $Id: lynx-lib.pl 177 2013-01-25 19:28:50Z gouzy $
#

#
# Utils
#

sub GetSoapdenovoCfg
{
my($ra_read_files,$outfile,$avg_ins)=@_;

	my $f_out = &GetStreamOut($outfile);
	if ( defined($avg_ins) )
	{
		print $f_out "[LIB]\n";
		if ( scalar(@$ra_read_files) % 2 != 0 )
		{
			for(my $i=0;$i<=$#$ra_read_files;$i++)
			{
				my ($format,$soapprefix) = ( $$ra_read_files[$i] =~ /\.fastq\.|\.fq\./ ) ? ('fastq','q') : ('fa','f');
				print $f_out "${soapprefix}=$$ra_read_files[$i]\n";
			}
		}
		else
		{
			print $f_out "avg_ins=$avg_ins\n";
			for(my $i=0;$i<=$#$ra_read_files;$i+=2)
			{
				my ($format,$soapprefix) = ( $$ra_read_files[$i] =~ /\.fastq\.|\.fq\./ ) ? ('fastq','q') : ('fa','f');
				print $f_out "${soapprefix}1=$$ra_read_files[$i]\n";
				print $f_out "${soapprefix}2=$$ra_read_files[$i+1]\n";
			}
		}
	}
	else
	{
		Carp::croak("GetSoapdenovoCfg//Fonction qui determine la taille de l'insert en fonction du nom de fichier ou d'un fichier de metadata n'est pas implementee (must be set to 0 for single ends\n");
	}
	$f_out->close;
}

#
#
#

sub GetReadLen
{
my($readfile)=@_;

	my $f_in      = &GetStreamIn($readfile);
	my $max       = 0;
	my $cpt       = 0; # si fastq cela sera en fait 200000 seq car je compte aussi les qualites pour faire simple
	my $max_reads = 400000;
	while(my $l=<$f_in>)
	{
		chomp($l);
		next if ( $l =~ /^[\>\+\@]/ );
		$max = length($l) if ( length($l) > $max );
		last if ( $cpt++ > $max_reads );
	}
	$f_in->close;
	return $max;
}

#
#
#

sub CheckExeAvailable
{
my (@a_exe)=@_;

	foreach my $exe (@a_exe)
	{
		&GetExePath($exe);
	}
}


#
#
#

sub System
{
my($cde)=@_;

	print STDERR "$cde\n" if (defined($main::DEBUG) && $main::DEBUG );
	system($cde);
	if ($? == -1) 
	{
		exit(0);
	}
	elsif ($? & 127) 
	{
		# plantage programme externe
		exit(0);
	}
	else 
	{
		#exit(1) if ( $? >> 8 && $main::DEBUG);
		return  $? >> 8;
	}
}

#
# Lynx basic command lines
#

#
#
#

sub LynxFilteringLinkingHSPs
{
my($reference,$m8in,$m8out,$avg_ins,$paired,$pci,$pcs,$lynx_rf_prg,$template_cfg)=@_;

	&DebugPrintParams('LynxFilteringLinkingHSPs',@_);

	my $o_cfg = New ParamParser($template_cfg);
	$o_cfg->Set('sequence_0_infile',abs_path($reference));
	$o_cfg->Set('sequence_0_what','all');
	$o_cfg->Set('sequence_1_insert_size',$avg_ins);
	$o_cfg->Set('sequence_1_source',''); # je veux la profondeur

	my @a_lstin = split(',',$m8in);
	if ( scalar(@a_lstin) > 1 )
	{
		$m8in="$a_lstin[0].mergedwith-end2.gz";
		&System("gzip -cd ".join(' ',@a_lstin)." | gzip -c > $m8in");
	}
	$o_cfg->Set('hsp_1_infile',abs_path($m8in));
	$o_cfg->Set('hsp_1_outfile',abs_path($m8out));
	$o_cfg->Set('hsp_1_minpci',$pci);
	$o_cfg->Set('hsp_1_minlen',$pcs);
	$o_cfg->Dump("$m8out.cfg");

	&System("$lynx_rf_prg --cfg $m8out.cfg");
}

sub LynxRefBasedContigOrderer
{
my($outdir,$in,$reference,$m8in,$pci,$lynx_prg,$template_cfg,$outprefix,$initwith)=@_;

	&DebugPrintParams('LynxRefBasedContigOrderer',@_);

	my $o_cfg = New ParamParser($template_cfg);
	$o_cfg->Set('sequence_0_infile',abs_path($in));
	$o_cfg->Set('sequence_1_infile',abs_path($reference));
	$o_cfg->Set('sequence_0_what','all');
	$o_cfg->Set('hsp_1_infile',$m8in);
	$o_cfg->Set('hsp_1_minpci',$pci);

	$o_cfg->Dump("$outdir/$outprefix.cfg");

	&System("cd $outdir ; $lynx_prg --cfg $outprefix.cfg --outprefix $outprefix --savelinks $outprefix.links.gz  2> $outprefix.stde");
}


sub LynxBasicScaffolding
{
my($outdir,$reference,$m8in,$avg_ins,$paired,$pci,$pcs,$lynx_prg,$template_cfg,$outprefix,$initwith,$steps,$params)=@_;

	&DebugPrintParams('LynxBasicScaffolding',@_);

	my $o_cfg = New ParamParser($template_cfg);

	$o_cfg->Set('scaffolder_basic_param',$params);
	$o_cfg->Set('scaffolder_steps',$o_cfg->Get('scaffolder_steps')." $steps") if ( defined($steps) );
	$o_cfg->Set('sequence_0_infile',abs_path($reference));
	$o_cfg->Set('sequence_0_what','all');
	$o_cfg->Set('sequence_1_insert_size',$avg_ins);
	$o_cfg->Set('sequence_1_source',''); # je veux la profondeur
	$o_cfg->Set('hsp_1_infile',abs_path($m8in));
	$o_cfg->Set('hsp_1_minpci',$pci);
	$o_cfg->Set('hsp_1_minlen',$pcs);

	$o_cfg->Dump("$outdir/$outprefix.cfg"); # pourquoi il y a deux fois outdir si je rajoute $outdir/$outprefix.cfg

	my $loadscaff = '';
	my $loadlinks = '';
	if ( defined($initwith) )
	{
		if ($initwith =~ /[\*\?]/ )
		{
			my @a_ls = `ls -t $initwith`;
			chomp(@a_ls);
			$initwith = $a_ls[0];
		}
		$initwith = abs_path($initwith);
		$loadscaff = " --loadscaff $initwith " if ( $initwith =~ /scaff$/ );
		$loadlinks = " --loadlinks $initwith " if ( $initwith =~ /links/ );
	}

	&System("cd $outdir ; $lynx_prg --cfg $outprefix.cfg --outprefix $outprefix $loadscaff $loadlinks 2> $outprefix.stde");
}


#
#
#


sub LynxClosingGapsDueToOvlContigs
{
my($outdir,$fastain,$lynx_prg,$template_cfg,$outprefix)=@_;

	&DebugPrintParams('LynxClosingGapsDueToOvlContigs',@_);

	my $o_cfg = New ParamParser($template_cfg);

	$o_cfg->Set('sequence_0_infile',$fastain);
	$o_cfg->Set('sequence_0_what','all');
	$o_cfg->Set('sequence_1_infile',$fastain);

	$o_cfg->Dump("$outdir/$outprefix.cfg");

	&System("cd $outdir ; $lynx_prg --cfg $outprefix.cfg --outprefix $outprefix --initfromfasta --checklinksbyalignment 2> $outprefix.stde");
}



#
#
#

sub CmdGlintWithEndsPatching
{
my($ra_read_files,$outdir,$db,$ncpus)=@_;

	&DebugPrintParams('CmdGlintWithEndsPatching',@_);
	my @a_glintoutfiles = ();
	my @a_filesforsoap  = ();
	for(my $i=0;$i<scalar(@$ra_read_files);$i++)
	{
		my $file = basename($$ra_read_files[$i]);
		my $outglint = "$outdir/glint.$file";
		my $f_reads = &GetStreamIn($$ra_read_files[$i]);
		my $ext = "";
		while(my $l=<$f_reads>)
		{
			if ($l =~ /^[>\@](\S+)/ )
			{
				my $readnm = $1;
				($ext) = $readnm =~ /(.+)\/(\d+)$/;
				last;
			}
		}
		$f_reads->close;
		if ( scalar(@$ra_read_files) == 1 )
		{
			if ( !defined($ext) || $ext eq '' )
			{
				Carp::croak("Error: a single file must contain IDs with read tags /1 and /2\n");
			}
		}
		my ($format,$soapprefix) = ( $$ra_read_files[$i] =~ /\.fastq\.|\.fq\./ ) ? ('fastq','q') : ('fa','f');
		if ( ! -e "$outglint.gz" || ! $main::DEBUG) 
		{
			&System("glint map $db $$ra_read_files[$i] -o $outglint.raw --step 2 --no-gap  --mmis 5 --best-score -C 0 --lrmin 80 -m 8 -a $ncpus --compress GZ --input-format $format ");
		}
		else
		{
			print STDERR "main::DEBUG, $outglint.raw.gz found, glint mapping skipped\n";
		}
		if ( !defined($ext) || $ext eq '' )
		{
			$ext = ( $i % 2 == 0 ) ? "/1" : "/2";
			print STDERR "Warning: adding $ext tag to $outglint.raw.gz\n";
			my $f_raw     = &GetStreamIn("$outglint.raw.gz");
			my $f_patchid = &GetStreamOut("$outglint.gz");
			while(my $l=<$f_raw>)
			{
					$l =~ s,^(\S+),$1$ext,;
					print $f_patchid $l;
			}
			$f_raw->close;
			$f_patchid->close;
		}
		else
		{
			&System("mv $outglint.raw.gz $outglint.gz");
		}
		push(@a_glintoutfiles,"$outglint.gz");
	}
	return \@a_glintoutfiles;
}


sub DebugPrintParams
{
my($sub,@a_params)=@_;

	return if ( ! $main::DEBUG );
	print STDERR "sub: $sub\n";
	for(my $i=0;$i<scalar(@a_params);$i++)
	{
		printf "%d\t$a_params[$i]\n",$i+1;
	}
}


1;
